// ----------------------------------------
//     
//   
// ----------------------------------------

#define STRICT
#include <windows.h>
#include <mem.h>
#include "dmenu.hpp"

//  
BOOL InitApp(HINSTANCE);
LRESULT CALLBACK _export WndProc(HWND, UINT, WPARAM, LPARAM);

//   
char const szClassName[]   = "DMenuClass";

//  
char const szWindowTitle[] = "Menu Demo";

//    
HMENU hmenu;

//   
HMENU hmenuFile; // "File"
HMENU hmenuEdit; // "Edit"
HMENU hmenuHelp; // "Help"

// =====================================
//  WinMain
// =====================================
#pragma argsused

int PASCAL
WinMain(HINSTANCE hInstance, 
	HINSTANCE hPrevInstance,
	LPSTR     lpszCmdLine, 
	int       nCmdShow)
{
  MSG  msg;   //     
  HWND hwnd;  //    

  //  
  if(!InitApp(hInstance))
      return FALSE;

  //     
  //   
  hwnd = CreateWindow(
    szClassName,         //   
    szWindowTitle,       //  
    WS_OVERLAPPEDWINDOW, //  
    CW_USEDEFAULT,       //    
    CW_USEDEFAULT,       // ,   
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    0,                   //   
    0,                   //  
    hInstance,           //  
    NULL);               //   
                         // 

  //     ,  
  if(!hwnd)
    return FALSE;

  //   
  ShowWindow(hwnd, nCmdShow);
  UpdateWindow(hwnd);

  //    
  while(GetMessage(&msg, 0, 0, 0))
  {
    DispatchMessage(&msg);
  }
  return msg.wParam;
}

// =====================================
//  InitApp
//    
// =====================================

BOOL
InitApp(HINSTANCE hInstance)
{
  ATOM aWndClass; //    
  WNDCLASS wc;    //   
                  //  

  //       
  memset(&wc, 0, sizeof(wc));

  wc.lpszMenuName  = NULL;
  wc.style         = 0;
  wc.lpfnWndProc   = (WNDPROC) WndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
  wc.lpszClassName = (LPSTR)szClassName;

  //  
  aWndClass = RegisterClass(&wc);

  return (aWndClass != 0);
}

// =====================================
//  WndProc
// =====================================

LRESULT CALLBACK _export
WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  switch (msg)
  {
    case WM_CREATE:
    {
      //     
      hmenu = CreateMenu();

      //      
      SetMenu(hwnd, hmenu);

      //     - "File"  "Help"
      hmenuFile = CreatePopupMenu();
      hmenuHelp = CreatePopupMenu();

      //     "File"
      AppendMenu(hmenuFile, MF_ENABLED | MF_STRING,
         CM_FILENEW,    "&New");
      AppendMenu(hmenuFile, MF_ENABLED | MF_STRING,
         CM_FILEOPEN,   "&Open");
      AppendMenu(hmenuFile, MF_GRAYED | MF_STRING,
         CM_FILECLOSE,  "&Close");
      AppendMenu(hmenuFile, MF_GRAYED | MF_STRING,
         CM_FILESAVE,   "&Save");
      AppendMenu(hmenuFile, MF_GRAYED | MF_STRING,
         CM_FILESAVEAS, "Save &as...");

      //   
      AppendMenu(hmenuFile, MF_SEPARATOR, 0, NULL);

      AppendMenu(hmenuFile, MF_GRAYED | MF_STRING,
         CM_FILEPRINT,         "&Print");
      AppendMenu(hmenuFile, MF_GRAYED | MF_STRING,
         CM_FILEPAGE_SETUP,    "Page Se&tup");
      AppendMenu(hmenuFile, MF_GRAYED | MF_STRING,
         CM_FILEPRINTER_SETUP, "P&rinter Setup");

      AppendMenu(hmenuFile, MF_SEPARATOR, 0, NULL);

      AppendMenu(hmenuFile, MF_DISABLED | MF_STRING,
         CM_FILEDEMO, "&Demo Version");

      AppendMenu(hmenuFile, MF_SEPARATOR, 0, NULL);

      AppendMenu(hmenuFile, MF_ENABLED | MF_STRING,
         CM_FILEEXIT, "E&xit");

      //    "Demo Version"
      CheckMenuItem(hmenuFile, CM_FILEDEMO,
         MF_BYCOMMAND | MF_CHECKED);

      //     "Help"
      AppendMenu(hmenuHelp, MF_GRAYED | MF_STRING,
         CM_HELPINDEX, "&Index\tF1");
      AppendMenu(hmenuHelp, MF_GRAYED | MF_STRING,
         CM_HELPKEYBOARD, "&Keyboard");
      AppendMenu(hmenuHelp, MF_GRAYED | MF_STRING,
         CM_HELPCOMMANDS, "&Commands");
      AppendMenu(hmenuHelp, MF_GRAYED | MF_STRING,
         CM_HELPPROCEDURES, "&Procedures");
      AppendMenu(hmenuHelp, MF_GRAYED | MF_STRING,
         CM_HELPUSING_HELP, "&Using help");

      AppendMenu(hmenuHelp, MF_SEPARATOR, 0, NULL);

      AppendMenu(hmenuHelp, MF_ENABLED | MF_STRING,
         CM_HELPABOUT, "&About...");

      //       
      AppendMenu(hmenu, MF_ENABLED | MF_POPUP,
         (UINT)hmenuFile, "&File");
      AppendMenu(hmenu, MF_ENABLED | MF_POPUP,
         (UINT)hmenuHelp, "&Help");

      //  
      DrawMenuBar(hwnd);

      //     "Edit" 
      // NULL.      ,   
      //   DestroyMenu   
      hmenuEdit = NULL;

      return 0;
    }

    case WM_COMMAND:
    {
      switch (wParam)
      {
        //   
	case CM_HELPUSING_HELP:
        case CM_HELPPROCEDURES:
        case CM_HELPCOMMANDS:
        case CM_HELPKEYBOARD:
        case CM_HELPINDEX:
        case CM_EDITPASTE:
        case CM_EDITCOPY:
        case CM_EDITCUT:
        case CM_EDITUNDO:
        case CM_FILEPRINTER_SETUP:
        case CM_FILEPAGE_SETUP:
        case CM_FILEPRINT:
        case CM_FILESAVEAS:
        case CM_FILESAVE:
	{
          //    
	  MessageBox(hwnd, "  ",
	          NULL, MB_OK);
          return 0;
	}

        //   "About..."   "Help"
	case CM_HELPABOUT:
	{
	  MessageBox(hwnd, " DMENU\n(C)  .., 1994",
		  szWindowTitle, MB_OK | MB_ICONINFORMATION);
          return 0;
	}

	//   "Open"  "New"   "File"
	case CM_FILEOPEN:
	case CM_FILENEW:
	{
          //    "Edit"
	  hmenuEdit = CreatePopupMenu();

          //     "Edit"
	  AppendMenu(hmenuEdit, MF_GRAYED  | MF_STRING,
	     CM_EDITUNDO,  "&Undo\tCtrl+Z");
	  AppendMenu(hmenuEdit, MF_ENABLED | MF_STRING,
	     CM_EDITCUT,   "&Cut\tCtrl+X");
	  AppendMenu(hmenuEdit, MF_ENABLED | MF_STRING,
	     CM_EDITCOPY,  "&Copy\tCtrl+C");
	  AppendMenu(hmenuEdit, MF_ENABLED | MF_STRING,
	     CM_EDITPASTE, "&Paste\tCtrl+V");

          //   "Edit"   "File"   "Help"
	  InsertMenu(hmenu, 1, MF_BYPOSITION | MF_ENABLED | MF_POPUP,
	     (UINT)hmenuEdit, "&Edit");

          //   "Save", "Save as..."  "Close"   "File"
	  EnableMenuItem(hmenuFile, CM_FILESAVE,
	     MF_ENABLED | MF_BYCOMMAND);
	  EnableMenuItem(hmenuFile, CM_FILESAVEAS,
	     MF_ENABLED | MF_BYCOMMAND);
	  EnableMenuItem(hmenuFile, CM_FILECLOSE,
	     MF_ENABLED | MF_BYCOMMAND);

          //   "New"  "Open"   "File"
	  EnableMenuItem(hmenuFile, CM_FILENEW,
	     MF_GRAYED | MF_BYCOMMAND);
	  EnableMenuItem(hmenuFile, CM_FILEOPEN,
	     MF_GRAYED | MF_BYCOMMAND);

          //  
	  DrawMenuBar(hwnd);
	  return 0;
	}

        //   "Close"   "File"
	case CM_FILECLOSE:
	{
          //    "Edit"
	  DestroyMenu(hmenuEdit);

	  hmenuEdit = NULL;

	  //     
          //  
	  RemoveMenu(hmenu, 1, MF_BYPOSITION);

          //   "Save", "Save as..."  "Close"   "File"
	  EnableMenuItem(hmenuFile, CM_FILESAVE,
	     MF_GRAYED | MF_BYCOMMAND);
	  EnableMenuItem(hmenuFile, CM_FILESAVEAS,
	     MF_GRAYED | MF_BYCOMMAND);
	  EnableMenuItem(hmenuFile, CM_FILECLOSE,
	     MF_GRAYED | MF_BYCOMMAND);

	  //   "New"  "Open"   "File"
	  EnableMenuItem(hmenuFile, CM_FILENEW,
	     MF_ENABLED | MF_BYCOMMAND);
	  EnableMenuItem(hmenuFile, CM_FILEOPEN,
	     MF_ENABLED | MF_BYCOMMAND);

	  //  
	  DrawMenuBar(hwnd);
	  return 0;
	}

	//   
        case CM_FILEEXIT:
        {
	  DestroyWindow(hwnd);
          return 0;
        }

        default:
          return 0;
      }
    }

    case WM_DESTROY:
    {
      //     "Edit",
      //  
      if(hmenuEdit != NULL)
      {
        DestroyMenu(hmenuEdit);
      }

      //    
      DestroyMenu(hmenuFile);
      DestroyMenu(hmenuHelp);
      DestroyMenu(hmenu);

      PostQuitMessage(0);
      return 0;
    }

    default:
      break;
  }
  return DefWindowProc(hwnd, msg, wParam, lParam);
}