// ================================================
//  WINHOOK
//   
//  Microsoft Windows
//    DLL- kbhook.dll
// ================================================

#define STRICT
#include <windows.h>
#include <windowsx.h>
#include <dos.h>
#include <mem.h>

#include "kbhook.hpp"

// ----------------------------------------------------
//  
// ----------------------------------------------------

extern "C"
void WINAPI _export SetKbHook(HWND hwnd);

extern "C"
void WINAPI _export RemoveKbHook(void);

BOOL InitApp(HINSTANCE);
LRESULT CALLBACK _export WndProc(HWND, UINT, WPARAM, LPARAM);

// ----------------------------------------------------
//  
// ----------------------------------------------------

char const szClassName[]   = "WINHOOKAppClass";
char const szWindowTitle[] = "WINHOOK Application";

TEXTMETRIC tm;
int cxChar, cyChar;
RECT rc;
static BOOL bCyrillic = FALSE;

// =====================================
//  WinMain
// =====================================
#pragma argsused

int PASCAL
WinMain(HINSTANCE hInstance,
	HINSTANCE hPrevInstance,
	LPSTR     lpszCmdLine,
	int       nCmdShow)
{
  MSG  msg;   //     
  HWND hwnd;  //    

  //      
  if(hPrevInstance)
    return 0;

  //   
  if(!InitApp(hInstance))
      return FALSE;

  //    Desktop.
  //      
  //       
  GetWindowRect(GetDesktopWindow(), &rc);

  //     
  //    ,  
  //    .
  //     
  //     
  hwnd = CreateWindow(
    szClassName, szWindowTitle, 
    WS_POPUPWINDOW | WS_THICKFRAME,
    100, 100,
    100, 100,
    0, 0, hInstance, NULL); 

  if(!hwnd)
    return FALSE;

  //     
  //      
  // , . .   
  //     
  SetWindowPos(hwnd, HWND_TOPMOST,
    rc.right  - cxChar * 15,
    rc.bottom - cyChar * 3,
    cxChar * 10, cyChar * 2, 0);

  //     
  ShowWindow(hwnd, nCmdShow);
  UpdateWindow(hwnd);

  //    
  while(GetMessage(&msg, 0, 0, 0))
  {
    DispatchMessage(&msg);
  }
  return msg.wParam;
}

// =====================================
//  InitApp
// =====================================

BOOL
InitApp(HINSTANCE hInstance)
{
  ATOM aWndClass; //    
  WNDCLASS wc;    //   
                  //  

  memset(&wc, 0, sizeof(wc));
  wc.style         = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc   = (WNDPROC) WndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon   = LoadIcon(NULL, IDI_APPLICATION);
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
  wc.lpszMenuName  = (LPSTR)NULL;
  wc.lpszClassName = (LPSTR)szClassName;

  aWndClass = RegisterClass(&wc);
  return (aWndClass != 0);
}

// =====================================
//  WndProc
// =====================================

LRESULT CALLBACK _export
WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  HDC        hdc;
  PAINTSTRUCT ps;

  switch (msg)
  {
    case WM_CREATE:
    {
      //  
      SetKbHook(hwnd);

      //   
      hdc = GetDC(hwnd);

      //      
      SelectObject(hdc, GetStockObject(SYSTEM_FIXED_FONT));

      //   
      //   ,  
      //  
      GetTextMetrics(hdc, &tm);

      //    
      //   
      cxChar = tm.tmMaxCharWidth;

      //     
      //   
      cyChar = tm.tmHeight + tm.tmExternalLeading;

      ReleaseDC(hwnd, hdc);
      return 0;
    }

    //    
    // ,   , 
    //    WM_NCHITTEST
    case WM_NCHITTEST:
    {
      long lRetVal;

      //   DefWindowProc  
      //   
      lRetVal = DefWindowProc(hwnd, msg, wParam, lParam);

      //       
      //   , 
      //    , 
      //  ,  
      //  DefWindowProc
      if(lRetVal == HTLEFT        ||
	 lRetVal == HTRIGHT       ||
	 lRetVal == HTTOP         ||
	 lRetVal == HTBOTTOM      ||
	 lRetVal == HTBOTTOMRIGHT ||
	 lRetVal == HTTOPRIGHT    ||
	 lRetVal == HTTOPLEFT     ||
	 lRetVal == HTBOTTOMLEFT)
      {
	return lRetVal;
      }

      //     
      // HTCAPTION,  
      //  .
      else
      {
	return HTCAPTION;
      }
    }

    case WM_DESTROY:
    {
      //    
      //  
      RemoveKbHook();

      PostQuitMessage(0);
      return 0;
    }

    //     DLL-
    //    
    case WM_KBHOOK:
    {
      //   
      bCyrillic = (BOOL)wParam;

      //   
      MessageBeep(0);

      //   
      InvalidateRect(hwnd, NULL, FALSE);
      return 0;
    }

    case WM_PAINT:
    {
      BYTE   szBuf[10];
      RECT    rc;

      //   
      hdc = BeginPaint(hwnd, &ps);

      //      
      SelectObject(hdc, GetStockObject(SYSTEM_FIXED_FONT));

      //     
      GetClientRect(hwnd, &rc);

      //      
      //    
      //   
      if(bCyrillic)
        lstrcpy(szBuf, (LPCSTR)"CYRILLIC");
      else
        lstrcpy(szBuf, (LPCSTR)"DEFAULT ");

      //     
      DrawText(hdc, (LPSTR)szBuf, 8, &rc,
	       DT_CENTER | DT_VCENTER | DT_NOCLIP | DT_SINGLELINE);

      EndPaint(hwnd, &ps);
    }
  }
  return DefWindowProc(hwnd, msg, wParam, lParam);
}

