// ----------------------------------------
//  DASHLINE
//    LineDDA
// ----------------------------------------

#define STRICT
#include <windows.h>
#include <windowsx.h>
#include <mem.h>

//  
BOOL InitApp(HINSTANCE);
LRESULT CALLBACK _export WndProc(HWND, UINT, WPARAM, LPARAM);
void CALLBACK _export LineProc(int xPos, int yPos, LPSTR lphdc);

//   
char const szClassName[]   = "DashLineClass";

//  
char const szWindowTitle[] = "Dash Line";

//   
HINSTANCE hInst;

// =====================================
//  WinMain
// =====================================
#pragma argsused

int PASCAL
WinMain(HINSTANCE hInstance, 
	HINSTANCE hPrevInstance,
	LPSTR     lpszCmdLine, 
	int       nCmdShow)
{
  MSG  msg;   //     
  HWND hwnd;  //    

  //  
  if(!InitApp(hInstance))
      return FALSE;

  //    
  hInst = hInstance;

  //    
  hwnd = CreateWindow(
    szClassName,         //   
    szWindowTitle,       //  
    WS_OVERLAPPEDWINDOW, //  
    CW_USEDEFAULT,       //    
    CW_USEDEFAULT,       // ,   
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    0, 0, hInstance, NULL);      
                       
  //     ,  
  if(!hwnd)
    return FALSE;

  //   
  ShowWindow(hwnd, nCmdShow);
  UpdateWindow(hwnd);

  //    
  while(GetMessage(&msg, 0, 0, 0))
  {
    DispatchMessage(&msg);
  }
  return msg.wParam;
}

// =====================================
//  InitApp
//    
// =====================================

BOOL
InitApp(HINSTANCE hInstance)
{
  ATOM aWndClass; //    
  WNDCLASS wc;    //   
                  //  
  //       
  memset(&wc, 0, sizeof(wc));

  wc.lpszMenuName  = NULL;
  wc.style         = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc   = (WNDPROC) WndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
  wc.lpszClassName = (LPSTR)szClassName;

  //  
  aWndClass = RegisterClass(&wc);

  return (aWndClass != 0);
}

// =====================================
//  WndProc
// =====================================

LRESULT CALLBACK _export
WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  HDC hdc;
  PAINTSTRUCT ps;
  static FARPROC lpfnLineProc;
  HPEN hpen, hpenOldPen;

  switch (msg)
  {
    case WM_CREATE:
    {
      //     LineProc 
      lpfnLineProc = MakeProcInstance((FARPROC)LineProc, hInst);
      return 0;
    }

    //   
    case WM_PAINT:
    {
      RECT rc;

      //    
      //      
      hdc = BeginPaint(hwnd, &ps);

      //    3   
      //     
      hpen = CreatePen(PS_SOLID, 3, RGB(0, 0, 0));
      hpenOldPen = SelectPen(hdc, hpen);

      //    ,
      //   

      LineDDA(50, 50, 300, 50,
        (LINEDDAPROC)lpfnLineProc, (DWORD)(LPSTR)&hdc);

      LineDDA(50, 50, 300, 100,
        (LINEDDAPROC)lpfnLineProc, (DWORD)(LPSTR)&hdc);

      LineDDA(50, 50, 50, 100,
        (LINEDDAPROC)lpfnLineProc, (DWORD)(LPSTR)&hdc);

      LineDDA(50, 100, 300, 100,
        (LINEDDAPROC)lpfnLineProc, (DWORD)(LPSTR)&hdc);

      LineDDA(300, 50, 300, 100,
        (LINEDDAPROC)lpfnLineProc, (DWORD)(LPSTR)&hdc);

      //      
      SelectPen(hdc, hpenOldPen);
      DeletePen(hpen);

      //   
      EndPaint(hwnd, &ps);
      return 0;
    }

    case WM_DESTROY:
    {
      //    LineProc
      FreeProcInstance(lpfnLineProc);

      PostQuitMessage(0);
      return 0;
    }

    default:
      break;
  }
  return DefWindowProc(hwnd, msg, wParam, lParam);
}

// --------------------------------------------------
//  LineProc     
// --------------------------------------------------
void CALLBACK _export
LineProc(int xPos, int yPos, LPSTR lphdc)
{
    //  
    static short cSpaces = 1;

    //      
    //  
    if(cSpaces == 1)
    {
      MoveToEx(*(HDC FAR*) lphdc, xPos, yPos, NULL);
      cSpaces++;
    }

    //      
    else if(cSpaces == 10)
    {
      LineTo(*(HDC FAR*) lphdc, xPos, yPos);
      cSpaces++;
    }

    //     
    //      
    else if(cSpaces == 20)
    {
      MoveToEx(*(HDC FAR*) lphdc, xPos, yPos, NULL);
      cSpaces = 1;
    }
    else
        cSpaces++;
}
