// ----------------------------------------
//  FONTVIEW
//  
// ----------------------------------------

#define STRICT
#include <windows.h>
#include <windowsx.h>
#include <commdlg.h>
#include <mem.h>
#pragma hdrstop

#include "fontview.hpp"

//  
BOOL InitApp(HINSTANCE);
LRESULT CALLBACK _export WndProc(HWND, UINT, WPARAM, LPARAM);
BOOL GetFont(HWND hWnd, LOGFONT *lf, CHOOSEFONT *cf);

//   
char const szClassName[]   = "FontViewClass";

//  
char const szWindowTitle[] = "Font Viewer";

//    
short cxClient, cyClient;

//   
HINSTANCE hInst;

//   
char szChars[] = ": AaBbCcDdEeFfGg ";
char szBuf[256];

//     
int nOrientation = 0;

// =====================================
//  WinMain
// =====================================
#pragma argsused

int PASCAL
WinMain(HINSTANCE hInstance, 
	HINSTANCE hPrevInstance,
	LPSTR     lpszCmdLine, 
	int       nCmdShow)
{
  MSG  msg;   //     
  HWND hwnd;  //    

  //  
  if(!InitApp(hInstance))
      return FALSE;

  hInst = hInstance;

  //     
  //   
  hwnd = CreateWindow(
    szClassName,         //   
    szWindowTitle,       //  
    WS_OVERLAPPEDWINDOW, //  
    CW_USEDEFAULT,       //    
    CW_USEDEFAULT,       // ,   
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    0, 0, hInstance, NULL);      
                       
  //     ,  
  if(!hwnd)
    return FALSE;

  //   
  ShowWindow(hwnd, nCmdShow);
  UpdateWindow(hwnd);

  //    
  while(GetMessage(&msg, 0, 0, 0))
  {
    DispatchMessage(&msg);
  }
  return msg.wParam;
}

// =====================================
//  InitApp
//    
// =====================================

BOOL
InitApp(HINSTANCE hInstance)
{
  ATOM aWndClass; //    
  WNDCLASS wc;    //   
                  //  

  //       
  memset(&wc, 0, sizeof(wc));

  //   
  wc.lpszMenuName  = "APP_MENU";

  wc.style         = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc   = (WNDPROC) WndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
  wc.lpszClassName = (LPSTR)szClassName;

  //  
  aWndClass = RegisterClass(&wc);

  return (aWndClass != 0);
}

// =====================================
//  WndProc
// =====================================

LRESULT CALLBACK _export
WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  HDC hdc;
  PAINTSTRUCT ps;
  static CHOOSEFONT cf;
  static LOGFONT lf;
  static HFONT hfont, hfOldFont;;

  switch (msg)
  {
    //     
    //      
    case WM_SIZE:
    {
      cxClient = LOWORD(lParam);
      cyClient = HIWORD(lParam);
      return 0;
    }

    //   
    case WM_PAINT:
    {
      //    
      //      
      hdc = BeginPaint(hwnd, &ps);

      //    
      lf.lfOrientation = lf.lfEscapement = nOrientation;

      //     
      //  LOGFONT
      hfont = CreateFontIndirect(&lf);

      if(hfont)
      {
        //     
        hfOldFont = SelectFont(hdc, hfont);

        //   
        GetTextFace(hdc, 80, szBuf);

        //     
        lstrcat(szBuf, szChars);

        //   
        SetTextColor(hdc, cf.rgbColors);

        //  ,   
        TextOut(hdc, cxClient/2, cyClient/2,
          szBuf, lstrlen(szBuf));

        //   
        SelectFont(hdc, hfOldFont);

        //     
        DeleteFont(hfont);
      }

      //   
      EndPaint(hwnd, &ps);
      return 0;
    }

    //    
    case WM_COMMAND:
    {
      switch (wParam)
      {
        //      
        case CM_FONTSEL:
        {
          //      
          // LOGFONT  
          memset(&lf, 0, sizeof(LOGFONT));

          //  
          if(GetFont(hwnd, &lf, &cf))
          {
            //  
            InvalidateRect(hwnd, NULL, TRUE);
          }
          return 0;
        }

        //  ,  
        case CM_FDECOR:
        {
          memset(&lf, 0, sizeof(LOGFONT));
          lf.lfPitchAndFamily = FF_DECORATIVE;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_FMODERN:
        {
          memset(&lf, 0, sizeof(LOGFONT));
          lf.lfPitchAndFamily = FF_MODERN;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_FROMAN:
        {
          memset(&lf, 0, sizeof(LOGFONT));
          lf.lfPitchAndFamily = FF_ROMAN;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_FSCRIPT:
        {
          memset(&lf, 0, sizeof(LOGFONT));
          lf.lfPitchAndFamily = FF_SCRIPT;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_FSWISS:
        {
          memset(&lf, 0, sizeof(LOGFONT));
          lf.lfPitchAndFamily = FF_SWISS;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }

        //    
        case CM_FONT00:
        {
          nOrientation = 0;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_FONT30:
        {
          nOrientation = 300;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_FONT45:
        {
          nOrientation = 450;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_FONT90:
        {
          nOrientation = 900;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_FONT180:
        {
          nOrientation = 1800;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_FONT270:
        {
          nOrientation = 2700;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_FONT360:
        {
          nOrientation = 3600;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }

        case CM_HELPABOUT:
        {
          MessageBox(hwnd,
            "Font Viewer, v.1.0\n"
            "(C) Frolov A.V., 1994",
	    "About FONTVIEW", MB_OK | MB_ICONINFORMATION);
          return 0;
        }

        //   
        case CM_FILEEXIT:
        {
          DestroyWindow(hwnd);
          return 0;
        }

        default:
          return 0;
      }
    }

    case WM_DESTROY:
    {
      PostQuitMessage(0);
      return 0;
    }

    default:
      break;
  }
  return DefWindowProc(hwnd, msg, wParam, lParam);
}

// =====================================
//  GetFont
// =====================================

BOOL GetFont(HWND hWnd, LOGFONT *lf, CHOOSEFONT *cf)
{
  LPSTR szFontStyle[LF_FACESIZE];

  //      
  // ,    
  //  
  memset(cf, 0, sizeof(CHOOSEFONT));

  //  
  cf->lStructSize = sizeof(CHOOSEFONT);

  //  
  cf->hwndOwner = hWnd;

  //    LOGFONT 
  cf->lpLogFont = lf;

  // ,     
  cf->Flags = CF_SCREENFONTS | CF_USESTYLE
            | CF_EFFECTS;

  //  
  cf->lCustData = 0L;

  //  
  cf->rgbColors = RGB(0,0,0);

  //    
  cf->lpfnHook = (FARPROC)NULL;

  //     
  cf->lpTemplateName = (LPSTR)NULL;

  //   
  cf->hInstance = hInst;

  //  
  cf->lpszStyle = (LPSTR)szFontStyle;

  //  
  cf->nFontType = SCREEN_FONTTYPE;

  //     
  //  
  cf->nSizeMin = 0;
  cf->nSizeMax = 0;

  //    
  return ChooseFont(cf);
} 




