// ----------------------------------------
//  LINER
//   
//   
// ----------------------------------------

#define STRICT
#include <windows.h>
#include <windowsx.h>
#include <mem.h>
#pragma hdrstop

#include "liner.hpp"

//  
BOOL InitApp(HINSTANCE);
LRESULT CALLBACK _export WndProc(HWND, UINT, WPARAM, LPARAM);
void DrawLines(HDC);
void DrawPolyline(HDC);
void DrawArc(HDC);
void DrawRectangles(HDC);
void DrawPolygon(HDC);
void DrawEllipse(HDC);
void DrawCircle(HDC);
void DrawPie(HDC);
void DrawChord(HDC);

//   
char const szClassName[]   = "LinerClass";

//  
char const szWindowTitle[] = "Liner";

//    
short cxClient, cyClient;

//     "Draw"
int nFigures = 0;

//  
int nBkMode = OPAQUE;

//   
int nROP2 = R2_COPYPEN;

// =====================================
//  WinMain
// =====================================
#pragma argsused

int PASCAL
WinMain(HINSTANCE hInstance, 
	HINSTANCE hPrevInstance,
	LPSTR     lpszCmdLine, 
	int       nCmdShow)
{
  MSG  msg;   //     
  HWND hwnd;  //    

  //  
  if(!InitApp(hInstance))
      return FALSE;

  //     
  //   
  hwnd = CreateWindow(
    szClassName,         //   
    szWindowTitle,       //  
    WS_OVERLAPPEDWINDOW, //  
    CW_USEDEFAULT,       //    
    CW_USEDEFAULT,       // ,   
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    0, 0, hInstance, NULL);      
                       
  //     ,  
  if(!hwnd)
    return FALSE;

  //   
  ShowWindow(hwnd, nCmdShow);
  UpdateWindow(hwnd);

  //    
  while(GetMessage(&msg, 0, 0, 0))
  {
    DispatchMessage(&msg);
  }
  return msg.wParam;
}

// =====================================
//  InitApp
//    
// =====================================

BOOL
InitApp(HINSTANCE hInstance)
{
  ATOM aWndClass; //    
  WNDCLASS wc;    //   
                  //  

  //       
  memset(&wc, 0, sizeof(wc));

  //   
  wc.lpszMenuName  = "APP_MENU";

  wc.style         = CS_HREDRAW | CS_VREDRAW;

  wc.lpfnWndProc   = (WNDPROC) WndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)GetStockObject(LTGRAY_BRUSH);
  wc.lpszClassName = (LPSTR)szClassName;

  //  
  aWndClass = RegisterClass(&wc);

  return (aWndClass != 0);
}

// =====================================
//  WndProc
// =====================================

LRESULT CALLBACK _export
WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  HDC hdc;
  PAINTSTRUCT ps;

  switch (msg)
  {
    //     
    //      
    case WM_SIZE:
    {
      cxClient = LOWORD(lParam);
      cyClient = HIWORD(lParam);
      return 0;
    }

    //   
    case WM_PAINT:
    {
      HBRUSH hbrush, hbrushOldBrush;
      int nCapHeight, nFrameHeight;

      //     

      //    
      //  
      hdc = GetWindowDC(hwnd);

      //    
      nCapHeight = GetSystemMetrics(SM_CYCAPTION);

      //    
      nFrameHeight = GetSystemMetrics(SM_CYFRAME);

      //    
      hbrush = CreateSolidBrush(RGB(0,0xff,0));

      //     , 
      //   
      hbrushOldBrush = SelectBrush(hdc, hbrush);

      //       
      //  
      Rectangle(hdc,
        2*nCapHeight, nFrameHeight,
        4*nCapHeight, nCapHeight);

      //     
      hbrush = CreateSolidBrush(RGB(0xff,0,0));
      SelectBrush(hdc, hbrush);

      //    
      //  
      Ellipse(hdc,
        2*nCapHeight, nFrameHeight,
        4*nCapHeight, nCapHeight);

      //   
      SelectBrush(hdc, hbrushOldBrush);

      //   ,
      //     
      ReleaseDC(hwnd, hdc);

      //     

      //    
      //      
      hdc = BeginPaint(hwnd, &ps);

      //   
      //     
      //     
      // 
      SetMapMode(hdc, MM_LOMETRIC);
      SetViewportOrg(hdc, 0, cyClient);

      //    
      //  
      SetBkMode(hdc, nBkMode);
      SetROP2(hdc, nROP2);

      //     
      // nFigures     
      if(nFigures == CM_LINES)
      {
        DrawLines(hdc);      //  
      }
      else if(nFigures == CM_ARC)
      {
        DrawArc(hdc);        //  
      }
      else if(nFigures == CM_RECT)
      {
        DrawRectangles(hdc); // 
      }
      else if(nFigures == CM_POLYGON)
      {
        DrawPolygon(hdc);    // 
      }
      else if(nFigures == CM_ELLIPSE)
      {
        DrawEllipse(hdc);    // 
      }
      else if(nFigures == CM_CIRCLE)
      {
        DrawCircle(hdc);     // 
      }
      else if(nFigures == CM_PIE)
      {
        DrawPie(hdc);        //  
      }
      else if(nFigures == CM_CHORD)
      {
        DrawChord(hdc);      //  
      }
      else if(nFigures == CM_POLYLINE)
      {
        DrawPolyline(hdc);   //  
      }

      //   
      EndPaint(hwnd, &ps);
      return 0;
    }

    //    
    case WM_COMMAND:
    {
      switch (wParam)
      {
        case CM_HELPABOUT:
        {
          MessageBox(hwnd,
            "Liner, v.1.0\n"
            "Drawing Graphics Demo\n"
            "(C) Frolov A.V., 1994",
	    "About Liner", MB_OK | MB_ICONINFORMATION);
          return 0;
        }

        //   
        case CM_OPAQUE:
        {
          //   
          nBkMode = OPAQUE;

          //   
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }

        //   
        case CM_TRANSPARENT:
        {
          nBkMode = TRANSPARENT;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }

        //   
        case CM_R2_BLACK:
        {
          //    
          nROP2 = R2_BLACK;

          //   
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_R2_NOTMERGEPEN:
        {
          nROP2 = R2_NOTMERGEPEN;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_R2_MASKNOTPEN:
        {
          nROP2 = R2_MASKNOTPEN;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_R2_NOTCOPYPEN:
        {
          nROP2 = R2_NOTCOPYPEN;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_R2_MASKPENNOT:
        {
          nROP2 = R2_MASKPENNOT;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_R2_NOT:
        {
          nROP2 = R2_NOT;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_R2_XORPEN:
        {
          nROP2 = R2_XORPEN;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_R2_NOTMASKPEN:
        {
          nROP2 = R2_NOTMASKPEN;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_R2_MASKPEN:
        {
          nROP2 = R2_MASKPEN;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_R2_NOTXORPEN:
        {
          nROP2 = R2_NOTXORPEN;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_R2_NOP:
        {
          nROP2 = R2_NOP;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_R2_MERGENOTPEN:
        {
          nROP2 = R2_MERGENOTPEN;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_R2_COPYPEN:
        {
          nROP2 = R2_COPYPEN;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_R2_MERGEPENNOT:
        {
          nROP2 = R2_MERGEPENNOT;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_R2_MERGEPEN:
        {
          nROP2 = R2_MERGEPEN;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }
        case CM_R2_WHITE:
        {
          nROP2 = R2_WHITE;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }

        //    "Draw"
        case CM_LINES:
        {
          nFigures = CM_LINES;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }

        case CM_POLYLINE:
        {
          nFigures = CM_POLYLINE;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }

        case CM_ARC:
        {
          nFigures = CM_ARC;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }

        case CM_RECT:
        {
          nFigures = CM_RECT;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }

        case CM_POLYGON:
        {
          nFigures = CM_POLYGON;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }

        case CM_ELLIPSE:
        {
          nFigures = CM_ELLIPSE;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }

        case CM_CIRCLE:
        {
          nFigures = CM_CIRCLE;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }

        case CM_PIE:
        {
          nFigures = CM_PIE;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }

        case CM_CHORD:
        {
          nFigures = CM_CHORD;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }

        //   
        case CM_FILEEXIT:
        {
          DestroyWindow(hwnd);
          return 0;
        }

        default:
          return 0;
      }
    }

    case WM_DESTROY:
    {
      PostQuitMessage(0);
      return 0;
    }

    default:
      break;
  }
  return DefWindowProc(hwnd, msg, wParam, lParam);
}

