// ----------------------------------------
//  PALETTE
//    
// ----------------------------------------

#define STRICT
#include <windows.h>
#include <windowsx.h>
#include <mem.h>

//    
#define PALETTESIZE   256

//  
BOOL InitApp(HINSTANCE);
LRESULT CALLBACK _export WndProc(HWND, UINT, WPARAM, LPARAM);
void PaletteInfo(void);

//   
char const szClassName[]   = "PaletteClass";

//  
char const szWindowTitle[] = "Palette Demo";

//    
short cxClient, cyClient;

//  
HPALETTE hPal, hOldPalette;

//    
NPLOGPALETTE  pLogPal;

// =====================================
//  WinMain
// =====================================
#pragma argsused

int PASCAL
WinMain(HINSTANCE hInstance, 
	HINSTANCE hPrevInstance,
	LPSTR     lpszCmdLine, 
	int       nCmdShow)
{
  MSG  msg;   //     
  HWND hwnd;  //    

  //  
  if(!hPrevInstance)
    if(!InitApp(hInstance))
      return FALSE;

  //    
  PaletteInfo();

  //     
  //   
  hwnd = CreateWindow(
    szClassName,         //   
    szWindowTitle,       //  
    WS_OVERLAPPEDWINDOW, //  
    CW_USEDEFAULT,       //    
    CW_USEDEFAULT,       // ,   
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    0, 0, hInstance, NULL);      
                       
  //     ,  
  if(!hwnd)
    return FALSE;

  //   
  ShowWindow(hwnd, nCmdShow);
  UpdateWindow(hwnd);

  //    
  while(GetMessage(&msg, 0, 0, 0))
  {
    DispatchMessage(&msg);
  }
  return msg.wParam;
}

// =====================================
//  InitApp
//    
// =====================================

BOOL
InitApp(HINSTANCE hInstance)
{
  ATOM aWndClass; //    
  WNDCLASS wc;    //   
                  //  
  //       
  memset(&wc, 0, sizeof(wc));

  wc.lpszMenuName  = NULL;
  wc.style         = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc   = (WNDPROC) WndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
  wc.lpszClassName = (LPSTR)szClassName;

  //  
  aWndClass = RegisterClass(&wc);

  return (aWndClass != 0);
}

// =====================================
//  WndProc
// =====================================

LRESULT CALLBACK _export
WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  HDC hdc;
  PAINTSTRUCT ps;

  switch (msg)
  {
    case WM_CREATE:
    {
      int i;

      //    
      pLogPal = (NPLOGPALETTE) LocalAlloc(LMEM_FIXED,
        (sizeof (LOGPALETTE) +
        (sizeof (PALETTEENTRY) * (PALETTESIZE))));

      //   
      pLogPal->palVersion = 0x300;
      pLogPal->palNumEntries = PALETTESIZE;

      //   
      //  
      for (i=0;  i < 256;  i++)
      {
        pLogPal->palPalEntry[i].peRed   =
        pLogPal->palPalEntry[i].peGreen =
        pLogPal->palPalEntry[i].peBlue  = i;
        pLogPal->palPalEntry[i].peFlags = 0;
      }

      //    
      hPal = CreatePalette((LPLOGPALETTE) pLogPal);
      return 0;
    }

    //     
    //      
    case WM_SIZE:
    {
      cxClient = LOWORD(lParam);
      cyClient = HIWORD(lParam);
      return 0;
    }

    //   
    case WM_PAINT:
    {
      RECT rc;
      int i, nWidth;
      HBRUSH hBrush;

      //    
      //      
      hdc = BeginPaint(hwnd, &ps);

      //  
      hOldPalette = SelectPalette(hdc, hPal, FALSE);

      //   
      RealizePalette(hdc);

      //   
      nWidth = 2;
      rc.left   = rc.top = 0;
      rc.right  = nWidth;
      rc.bottom = cyClient;

      //  64   
      //  
      for (i=0;  i < 256;  i++)
      {
        //  .     
        //    ,  
        // 

        //    
//      hBrush = CreateSolidBrush (PALETTERGB (i, i, i));

        //    
        hBrush = CreateSolidBrush (PALETTEINDEX (i));

        //    
        FillRect (hdc, &rc, hBrush);

        //   
        rc.left   = rc.right;
        rc.right += nWidth;

        //  
        DeleteBrush(hBrush);
      }

      //   
      SelectPalette(hdc, hOldPalette, TRUE);

      //   
      EndPaint(hwnd, &ps);
      return 0;
    }

    //     
    //  .    
    //       
    //      
    case WM_PALETTECHANGED:
    {
       //     ,  
       //   WM_QUERYNEWPALETTE
       if (hwnd == (HWND) wParam)
         break;
    }

    //       
    //     
    //  
    case WM_QUERYNEWPALETTE:
    {
      HDC hdc;
      HPALETTE hOldPal;
      int nChanged;

      //    
      //  
      hdc = GetDC(hwnd);

      //    WM_QUERYNEWPALETTE
      //     ,
      //     WM_PALETTECHANGED -
      //  
      hOldPal = SelectPalette(hdc, hPal,
        (msg == WM_QUERYNEWPALETTE) ? FALSE : TRUE);

      //     
      //    
      nChanged = RealizePalette(hdc);
      SelectPalette(hdc, hOldPal, TRUE);

      //   
      ReleaseDC(hwnd, hdc);

      //    ,
      //  
      if(nChanged)
        InvalidateRect(hwnd, NULL, TRUE);

      return nChanged;
    }

    case WM_DESTROY:
    {
      //   
      //  
      DeletePalette(hPal);

      //  ,   
      LocalFree(pLogPal);

      PostQuitMessage(0);
      return 0;
    }

    default:
      break;
  }
  return DefWindowProc(hwnd, msg, wParam, lParam);
}

// --------------------------------------------------------
//  PaletteInfo
//     
// --------------------------------------------------------

void PaletteInfo(void)
{
  HDC hdc;
  int iPalSize, iRasterCaps;
  char szMsg[256];
  char szPal[20];

  //    
  //  
  hdc = GetDC(NULL);

  //     ,
  //   
  //    
  iPalSize = GetDeviceCaps(hdc, SIZEPALETTE);
  iRasterCaps = GetDeviceCaps(hdc, RASTERCAPS);

  // ,    
  if(iRasterCaps & RC_PALETTE)
  {
    iRasterCaps = TRUE;
    lstrcpy(szPal, "");
  }
  else
  {
    iRasterCaps = FALSE;
    lstrcpy(szPal, " ");
  }

  //   
  ReleaseDC(NULL, hdc);

  //     
  wsprintf(szMsg, " %s\n"
    "  : %d\n",
    (LPSTR)szPal, iPalSize);

  MessageBox(NULL, szMsg, "Palette Demo", MB_OK);
}