// ----------------------------------------
//  REGIONS
//    
// ----------------------------------------

#define STRICT
#include <windows.h>
#include <windowsx.h>
#include <mem.h>

//  
BOOL InitApp(HINSTANCE);
LRESULT CALLBACK _export WndProc(HWND, UINT, WPARAM, LPARAM);

//   
char const szClassName[]   = "RegionsClass";

//  
char const szWindowTitle[] = "Regions";

//    
short cxClient, cyClient;

//   
int cxChar, cyChar;

//      
char const szText[] =
 "  GDI  ,  "
 "      "
 " ,    ."
 "      "
 "       . "
 "     "
 " .       .";

// =====================================
//  WinMain
// =====================================
#pragma argsused

int PASCAL
WinMain(HINSTANCE hInstance, 
	HINSTANCE hPrevInstance,
	LPSTR     lpszCmdLine, 
	int       nCmdShow)
{
  MSG  msg;   //     
  HWND hwnd;  //    

  //  
  if(!InitApp(hInstance))
      return FALSE;

  //     
  //   
  hwnd = CreateWindow(
    szClassName,         //   
    szWindowTitle,       //  
    WS_OVERLAPPEDWINDOW, //  
    CW_USEDEFAULT,       //    
    CW_USEDEFAULT,       // ,   
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    0, 0, hInstance, NULL);      
                       
  //     ,  
  if(!hwnd)
    return FALSE;

  //   
  ShowWindow(hwnd, nCmdShow);
  UpdateWindow(hwnd);

  //    
  while(GetMessage(&msg, 0, 0, 0))
  {
    DispatchMessage(&msg);
  }
  return msg.wParam;
}

// =====================================
//  InitApp
//    
// =====================================

BOOL
InitApp(HINSTANCE hInstance)
{
  ATOM aWndClass; //    
  WNDCLASS wc;    //   
                  //  
  //       
  memset(&wc, 0, sizeof(wc));

  wc.lpszMenuName  = NULL;
  wc.style         = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc   = (WNDPROC) WndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
  wc.lpszClassName = (LPSTR)szClassName;

  //  
  aWndClass = RegisterClass(&wc);

  return (aWndClass != 0);
}

// =====================================
//  WndProc
// =====================================

LRESULT CALLBACK _export
WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  HDC hdc;
  PAINTSTRUCT ps;
  static TEXTMETRIC tm;
  static HRGN hrgn1, hrgn2, hrgn3, hrgnTemp, hrgnClip;
  HBRUSH hbrush;

  switch (msg)
  {
    case WM_CREATE:
    {
      //   
      hdc = GetDC(hwnd);
      GetTextMetrics(hdc, &tm);
      ReleaseDC(hwnd, hdc);

      //     
      cyChar = tm.tmHeight + tm.tmExternalLeading;
      cxChar = tm.tmAveCharWidth;

      //    
      hrgnClip = NULL;

      return 0;
    }

    //     
    //      ,
    //     
    case WM_SIZE:
    {
      cxClient = LOWORD(lParam);
      cyClient = HIWORD(lParam);

      //      ,
      //  
      if(hrgnClip)
        DeleteRgn(hrgnClip);

      //   
      hrgnClip = CreateEllipticRgn(0, 0, cxClient, cyClient);

      //   
      hrgnTemp = CreateEllipticRgn(0, 0, cxClient, cyClient);

      //   
      hrgn1 = CreateEllipticRgn(0, 0, cxClient, cyClient);

      //   
      hrgn2 = CreateEllipticRgn(cxClient/3, cyClient/3,
         2*(cxClient/3), 2*(cyClient/3));

      //  
      hrgn3 = CreateRectRgn(cxClient/20, cyClient/20,
         19*(cxClient/20), 19*(cyClient/20));

      //  
      UnionRgn(hrgnTemp, hrgn1, hrgn3);
      SubtractRgn(hrgnClip, hrgnTemp, hrgn2);

      //   
      DeleteRgn(hrgn1);
      DeleteRgn(hrgn2);
      DeleteRgn(hrgn3);
      DeleteRgn(hrgnTemp);
      return 0;
    }

    //   
    case WM_PAINT:
    {
      RECT rc;

      //    
      //      
      hdc = BeginPaint(hwnd, &ps);

      //     
      hbrush = CreateSolidBrush(RGB(0, 0xff, 0));

      //   
      FrameRgn(hdc, hrgnClip, hbrush, 2, 5);

      //     
      // 
      SelectClipRgn(hdc, hrgnClip);

      //   
      //    
      rc.left = cxChar;
      rc.top  = 0;
      rc.right  = cxClient - cxChar;
      rc.bottom = cyClient;

      //  
      DrawText(hdc, szText, lstrlen(szText), &rc,
        DT_LEFT | DT_WORDBREAK);

      //   
      EndPaint(hwnd, &ps);
      return 0;
    }

    case WM_DESTROY:
    {
      //   
      DeleteRgn(hrgnClip);

      PostQuitMessage(0);
      return 0;
    }

    default:
      break;
  }
  return DefWindowProc(hwnd, msg, wParam, lParam);
}

