// ----------------------------------------
//   -
// ----------------------------------------

#define STRICT
#include <windows.h>
#include <mmsystem.h>
#include <mem.h>
#include <stdlib.h>

#include "mcicdpl.hpp"

#define CD_EMPTY   0
#define CD_READY   1
#define CD_PLAYING 2
#define CD_PAUSED  3

//  
BOOL InitApp(HINSTANCE);
LRESULT CALLBACK _export WndProc(HWND, UINT, WPARAM, LPARAM);
void mciwioError(DWORD dwrc);
void Play(HWND hwnd, UINT nTrack);


//   
char const szClassName[]   = "MCICDP";

//  
char const szWindowTitle[] = "MCI CD Player";

HINSTANCE hInst;

DWORD dwrc;
UINT nTimerID;

MCI_OPEN_PARMS   MCIOpen;
MCI_SET_PARMS    MCISet;
MCI_STATUS_PARMS MCIStatus;
MCI_PLAY_PARMS   MCIPlay;

BOOL bMediaPresent = FALSE;
BOOL bPaused       = FALSE;
UINT nMode = 0;
UINT nCurTrack = 0;
UINT nTrackCnt = 0;

HWND hwndCurTrack = NULL;

// =====================================
//  WinMain
// =====================================
#pragma argsused

int PASCAL
WinMain(HINSTANCE hInstance,
	HINSTANCE hPrevInstance,
	LPSTR     lpszCmdLine, 
	int       nCmdShow)   
{
  MSG  msg;   //     
  HWND hwnd;  //    

  if(hPrevInstance)
    return FALSE;

  //  
  if(!InitApp(hInstance))
    return FALSE;

  hInst = hInstance;

  //    -
  MCIOpen.lpstrDeviceType = (LPSTR)MCI_DEVTYPE_CD_AUDIO;
  dwrc = mciSendCommand(NULL, MCI_OPEN,
    MCI_OPEN_TYPE | MCI_OPEN_TYPE_ID,
    (DWORD)(LPVOID)&MCIOpen);
  if(dwrc)
  {
    mciwioError(dwrc);
    return -1;
  }

  //   
  MCISet.dwTimeFormat = MCI_FORMAT_TMSF;
  dwrc = mciSendCommand(MCIOpen.wDeviceID, MCI_SET,
    MCI_SET_TIME_FORMAT,
    (DWORD)(LPVOID)&MCISet);
  if(dwrc)
  {
    mciwioError(dwrc);
    return -1;
  }

  //      
  hwnd = CreateDialog(hInstance, szClassName, 0, NULL);

  //     ,  
  if(!hwnd)
    return FALSE;

  //   ,  
  //     
  hwndCurTrack = GetDlgItem(hwnd, IDT_CURTRACK);

  //   
  ShowWindow(hwnd, nCmdShow);
  UpdateWindow(hwnd);

  //    
  while(GetMessage(&msg, NULL, 0, 0))
  {
    if((hwnd == 0) || (!IsDialogMessage(hwnd, &msg)))
      DispatchMessage(&msg);
  }
  return msg.wParam;
}

// =====================================
//  InitApp
//    
// =====================================

BOOL
InitApp(HINSTANCE hInstance)
{
  ATOM aWndClass; //    
  WNDCLASS wc;    //   
                  //  

  memset(&wc, 0, sizeof(wc));

  wc.style = 0;
  wc.lpfnWndProc    = (WNDPROC) WndProc;
  wc.cbClsExtra     = 0;
  wc.cbWndExtra     = DLGWINDOWEXTRA;
  wc.hInstance      = hInstance;
  wc.hIcon          = LoadIcon(hInstance, "APPICON");
  wc.hCursor        = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground  = NULL;
  wc.lpszMenuName   = (LPSTR)NULL;
  wc.lpszClassName  = (LPSTR)szClassName;

  //  
  aWndClass = RegisterClass(&wc);

  return (aWndClass != 0);
}

// =====================================
//  WndProc
// =====================================

LRESULT CALLBACK _export
WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  switch (msg)
  {
    // ---------------------------------------
    //   WM_CREATE
    // ---------------------------------------
    case WM_CREATE:
    {
      //  ,    
      //     CD
      nTimerID = SetTimer(hwnd, 1, 1000, NULL);

      return 0;
    }

    // ---------------------------------------
    //   WM_COMMAND
    // ---------------------------------------
    case WM_COMMAND:
    {
      switch(wParam)
      {
        //    
        case IDB_PLAY:
        {
          //     -,
          //  
          if(bMediaPresent)
            Play(hwnd, 1);
          return 0;
        }

        //  
        case IDB_STOP:
        {
          if(bMediaPresent)
          {
            bPaused = FALSE;
            nCurTrack = 0;
            mciSendCommand(MCIOpen.wDeviceID, MCI_STOP,
              NULL, NULL);
          }
          return 0;
        }

        //   
        case IDB_PAUSE:
        {
          if(bMediaPresent)
          {
            if(!bPaused)
            {
              bPaused = TRUE;
              mciSendCommand(MCIOpen.wDeviceID, MCI_PAUSE,
                NULL, NULL);
            }
          }
          return 0;
        }

        //   
        //  
        case IDB_RESUME:
        {
          if(bMediaPresent)
          {
            if(bPaused)
            {
              bPaused = FALSE;
              MCIPlay.dwCallback = (DWORD)hwnd;
              mciSendCommand(MCIOpen.wDeviceID, MCI_PLAY,
                MCI_NOTIFY, (DWORD)(LPVOID)&MCIPlay);
            }
          }
          return 0;
        }

        //    
        case IDB_NEXT:
        {
          if(bMediaPresent)
          {
            UINT nNewTrack;

            //    - ,
            //     .
            //   -   
            if(nCurTrack == nTrackCnt)
              nNewTrack = 1;
            else
              nNewTrack = nCurTrack + 1;

            Play(hwnd, nNewTrack);
          }
          return 0;
        }

        //     
        case IDB_PREV:
        {
          if(bMediaPresent)
          {
            UINT nNewTrack;

            //    - ,
            //   
            if(nCurTrack <= 1)
              nNewTrack = nTrackCnt;
            else
              nNewTrack = nCurTrack - 1;

            Play(hwnd, nNewTrack);
          }
          return 0;
        }

        //   
        case IDOK:
        case IDCANCEL:
        {
          SendMessage(hwnd, WM_CLOSE, 0, 0L);
          return 0;
        }

        //     
        //  
        case IDB_EJECT:
        {
          mciSendCommand(MCIOpen.wDeviceID, MCI_SET,
            MCI_SET_DOOR_OPEN, NULL);
          return 0;
        }
      }
    }

    // ---------------------------------------
    //   WM_TIMER
    // ---------------------------------------
    case WM_TIMER:
    {
      UINT nCurMode;

      //     ,   ,
      //     
      if(IsIconic(hwnd))
        return 0;

      //     CD
      MCIStatus.dwItem = MCI_STATUS_MODE;
      mciSendCommand(MCIOpen.wDeviceID, MCI_STATUS,
        MCI_STATUS_ITEM | MCI_WAIT,
        (DWORD)(LPVOID)&MCIStatus);

      // ,       
      if((MCIStatus.dwReturn == MCI_MODE_NOT_READY) ||
         (MCIStatus.dwReturn == MCI_MODE_OPEN))
      {
        //   
        nCurMode = CD_EMPTY;
      }
       else if((MCIStatus.dwReturn == MCI_MODE_STOP) &&
         bPaused)
      {
        //   
        nCurMode = CD_PAUSED;
      }
      else if(MCIStatus.dwReturn == MCI_MODE_PLAY)
      {  
        //     
        nCurMode = CD_PLAYING;
      }
      else
      {
        //  
        nCurMode = CD_READY;
      }

      //      
      //  ,    
      if(nMode != nCurMode)
      {
        nMode = nCurMode;
      }

      // ,   -
      MCIStatus.dwItem = MCI_STATUS_MEDIA_PRESENT;
      mciSendCommand(MCIOpen.wDeviceID, MCI_STATUS,
        MCI_STATUS_ITEM | MCI_WAIT,
        (DWORD)(LPVOID)&MCIStatus);

      //  - , 
      //   
      if((!bMediaPresent) && MCIStatus.dwReturn)
      {
        bMediaPresent = TRUE;
        bPaused = FALSE;
        nCurTrack = 0;

        MCIStatus.dwItem = MCI_STATUS_NUMBER_OF_TRACKS;
        mciSendCommand(MCIOpen.wDeviceID, MCI_STATUS,
          MCI_STATUS_ITEM | MCI_WAIT,
          (DWORD)(LPVOID)&MCIStatus);

        nTrackCnt = MCIStatus.dwReturn;
      }

      //  -  , 
      //       
      else if((bMediaPresent) && !MCIStatus.dwReturn)
      {
        bMediaPresent = FALSE;
        bPaused = FALSE;
        SetWindowText(hwndCurTrack, (LPSTR)"");
      }

      //      ,
      //    
      if(nCurMode == CD_PLAYING)
      {
        //   
        MCIStatus.dwItem = MCI_STATUS_POSITION;
        mciSendCommand(MCIOpen.wDeviceID, MCI_STATUS,
          MCI_STATUS_ITEM | MCI_WAIT,
          (DWORD)(LPVOID)&MCIStatus);

        //    ,  
        //       
        if(nCurTrack != (UINT)MCI_TMSF_TRACK(MCIStatus.dwReturn))
        {
          BYTE szBuf[20];
          nCurTrack = (UINT)MCI_TMSF_TRACK(MCIStatus.dwReturn);
          SetWindowText(hwndCurTrack, itoa(nCurTrack, szBuf, 10));
        }
      }
      return 0;
    }

    // ---------------------------------------
    //   MM_MCINOTIFY
    // ---------------------------------------
    case MM_MCINOTIFY:
    {
      if(wParam == MCI_NOTIFY_SUCCESSFUL)
      {
        if(bMediaPresent)
           Play(hwnd, 1);
      }
      return 0;
    }

    // ---------------------------------------
    //   WM_CLOSE
    // ---------------------------------------
    case WM_CLOSE:
    {
      DestroyWindow(hwnd);
      return 0;
    }

    // ---------------------------------------
    //   WM_DESTROY
    // ---------------------------------------
    case WM_DESTROY:
    {
      //    -
      dwrc = mciSendCommand(MCIOpen.wDeviceID, MCI_CLOSE,
        NULL, NULL);
      if(dwrc)
        mciwioError(dwrc);

      //  
      KillTimer(hwnd, nTimerID);

      PostQuitMessage(0);
      return 0;
    }
  }
  return DefDlgProc(hwnd, msg, wParam, lParam);
}

//-----------------------------------------------------
// mciwioError
//  
//-----------------------------------------------------
void mciwioError(DWORD dwrc)
{
  BYTE szBuf[MAXERRORLENGTH];

  if(mciGetErrorString(dwrc, (LPSTR)szBuf, MAXERRORLENGTH))
    MessageBox(NULL, szBuf,
      "MCIWAVE Error", MB_ICONEXCLAMATION);
  else
    MessageBox(NULL, " ",
      "MCIWAVE Error", MB_ICONEXCLAMATION);
}

//-----------------------------------------------------
// Play
//   
//-----------------------------------------------------
void Play(HWND hwnd, UINT nTrack)
{
  bPaused = FALSE;

  MCIPlay.dwCallback = (DWORD)hwnd;
  MCIPlay.dwFrom     = MCI_MAKE_TMSF(nTrack, 0, 0, 0);

  dwrc = mciSendCommand(MCIOpen.wDeviceID, MCI_PLAY,
    MCI_FROM | MCI_NOTIFY,
    (DWORD)(LPVOID)&MCIPlay);
  if(dwrc)
  {
    mciwioError(dwrc);
    return;
  }
}