// ----------------------------------------
//  WAVE
//    wav-   
// ----------------------------------------

#define STRICT
#include <windows.h>
#include <windowsx.h>
#include <mmsystem.h>
#include <mem.h>
#pragma hdrstop

#include "wave.hpp"
#include "waveio.hpp"

//  
#define BEEP_TIMER 1

//   
#define ID_SCROLL 10

//   
#define SCROLL_SIZE 400

//  
BOOL    InitApp(HINSTANCE);
LRESULT CALLBACK _export WndProc(HWND, UINT, WPARAM, LPARAM);
BOOL    WAVELoad(LPWAVEIOCB lpwiocb);

//  
HWAVEOUT hWaveOut;
WAVEIOCB waveiocbOut;
HWAVEIN  hWaveIn;
WAVEIOCB waveiocbIn;
int      nMode = MODE_STOP;
MMTIME   mmtimeIn, mmtimeOut;
BOOL     fNeedSave = FALSE;
BOOL     fFileLoaded = FALSE;
int      nPosition;
HWND     hScroll;
char const szClassName[]   = "WaveClass";
char const szWindowTitle[] = "Wave Player/Recorder";
HINSTANCE  hInst;

// =====================================
//  WinMain
// =====================================
#pragma argsused

int PASCAL
WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
        LPSTR lpszCmdLine, int nCmdShow)
{
  MSG  msg;   //     
  HWND hwnd;  //    

  if(hPrevInstance)
    return FALSE;

  if(!InitApp(hInstance))
    return FALSE;

  hInst = hInstance;

  hwnd = CreateWindow(
    szClassName,         //   
    szWindowTitle,       //  
    WS_OVERLAPPEDWINDOW, //  
    CW_USEDEFAULT,       //    
    CW_USEDEFAULT,       
    450, 120, 0, 0, hInstance, NULL);
                       
  if(!hwnd)
    return FALSE;

  ShowWindow(hwnd, nCmdShow);
  UpdateWindow(hwnd);

  while(GetMessage(&msg, 0, 0, 0))
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }
  return msg.wParam;
}

// =====================================
//  InitApp
//    
// =====================================

BOOL
InitApp(HINSTANCE hInstance)
{
  ATOM aWndClass; //    
  WNDCLASS wc;    //   

  memset(&wc, 0, sizeof(wc));
  wc.lpszMenuName  = "APP_MENU";
  wc.style         = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc   = (WNDPROC) WndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon(hInstance, "APPICON");
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
  wc.lpszClassName = (LPSTR)szClassName;

  aWndClass = RegisterClass(&wc);
  return (aWndClass != 0);
}

// =====================================
//  WndProc
// =====================================

LRESULT CALLBACK _export
WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  HDC hdc;
  PAINTSTRUCT ps;
  int rc;

  switch (msg)
  {

// ------------------------------------------------------------
// WM_CREATE
//    
// ------------------------------------------------------------
    case WM_CREATE:
    {
       nMode       = MODE_STOP;
       fNeedSave   = FALSE;
       fFileLoaded = FALSE;
       hWaveIn     = NULL;
       hWaveOut    = NULL;

       //  
       SetTimer(hwnd, BEEP_TIMER, 100, NULL);

       //   
       hScroll = CreateWindow("scrollbar", NULL,
         WS_CHILD | WS_VISIBLE | SBS_HORZ,
         10, 40, SCROLL_SIZE, 15, hwnd,
         (HMENU) ID_SCROLL, hInst, NULL);

       //   
       nPosition = 0;

       //    
       //    
       SetScrollRange(hScroll, SB_CTL, 1, SCROLL_SIZE, TRUE);

       //  
       SetScrollPos(hScroll, SB_CTL, nPosition, TRUE);
       return 0;
    }

// ------------------------------------------------------------
// WM_PAINT
//   
// ------------------------------------------------------------
    case WM_PAINT:
    {
      //    
      //      
      hdc = BeginPaint(hwnd, &ps);

      //    
      if(nMode == MODE_STOP)
        TextOut(hdc, 10, 10, "", 11);
      else if(nMode == MODE_RECORDING)
        TextOut(hdc, 10, 10, " ...", 14);
      else if(nMode == MODE_PLAYING)
        TextOut(hdc, 10, 10, " ...", 20);
      else if(nMode == MODE_RECORDINGPAUSED)
        TextOut(hdc, 10, 10, " ", 18);
      else if(nMode == MODE_PLAYINGPAUSED)
        TextOut(hdc, 10, 10, " ", 24);
      else
        TextOut(hdc, 10, 10, " !", 19);

      //   
      EndPaint(hwnd, &ps);
      return 0;
    }

// ------------------------------------------------------------
// WM_COMMAND
//    
// ------------------------------------------------------------
    case WM_COMMAND:
    {
      switch (wParam)
      {
        // -------------------------------------------------
        //  "About"  "Help"
        // -------------------------------------------------
        case CM_HELPABOUT:
        {
          MessageBox(hwnd,
            "Wave, v.1.0\nWave Player/Recorder\n"
            "(C) Frolov A.V., 1994",
	    "About Wave", MB_OK | MB_ICONINFORMATION);
          return 0;
        }

        // -------------------------------------------------
        //  "Open"  "File"
        // -------------------------------------------------
        case CM_FILEOPEN:
        {
          //     ,  WAVE 
          //  
          if(fFileLoaded)
          {
            if(hWaveOut)
            {
              //   
              rc=waveOutReset(hWaveOut);
              if(rc) wioOutError(rc);

              //   
              rc=waveOutClose(hWaveOut);
              if(rc) wioOutError(rc);
            }

            //  ,   
            GlobalFreePtr(waveiocbOut.lpWaveHdr);
            GlobalFreePtr(waveiocbOut.lpData);
            GlobalFreePtr(waveiocbOut.lpFmt);
          }

          //   
          if(!WAVELoad(&waveiocbOut))
            return 0;

          //    
          fFileLoaded = TRUE;
          return 0;
        }

        // -------------------------------------------------
        //  "Play!"
        //   wav-
        // -------------------------------------------------
        case CM_CTLPLAY:
        {
          if(nMode == MODE_STOP)
          {
            //      ,
            //   
            if((fFileLoaded == TRUE) && (nMode != MODE_PLAYING))
            {
              //  
              nMode = MODE_PLAYING;

              //     ,
              //   
              InvalidateRect(hwnd, NULL, TRUE);

              //  
              rc=wioPlay(&waveiocbOut, hwnd);
              if(rc) wioOutError(rc);
            }
          }
          return 0;
        }

        // -------------------------------------------------
        //  "Record!"
        //  wav-
        // -------------------------------------------------
        case CM_CTLRECORD:
        {
          //      
          if(nMode == MODE_STOP)
          {
            nMode = MODE_RECORDING;
            InvalidateRect(hwnd, NULL, TRUE);

            //    
            fNeedSave = TRUE;

            //  
            wioRecord(&waveiocbIn, hwnd);
          }
          return 0;
        }

        // -------------------------------------------------
        //  "Stop!"
        //     wav-
        // -------------------------------------------------
        case CM_CTLSTOP:
        {
          if(nMode == MODE_RECORDING || nMode == MODE_RECORDINGPAUSED)
          {
            if(hWaveIn)
            {
              //  
              rc=waveInReset(hWaveIn);
              if(rc) wioInError(rc);

              //   
              rc=waveInClose(hWaveIn);
              if(rc) wioInError(rc);
              hWaveIn = 0;
            }
          }

          else if(nMode == MODE_PLAYING || nMode == MODE_PLAYINGPAUSED)
          {
            if(hWaveOut)
            {
              //  
              rc=waveOutReset(hWaveOut);
              if(rc) wioOutError(rc);

              //   
              rc=waveOutClose(hWaveOut);
              if(rc) wioOutError(rc);
              hWaveOut = 0;
            }
          }

          //      
          nPosition = 0;
          SetScrollPos(hScroll, SB_CTL, nPosition, TRUE);

          //  
          nMode = MODE_STOP;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }

        // -------------------------------------------------
        //  "Pause!"
        //    
        //    wav-
        // -------------------------------------------------
        case CM_CTLPAUSE:
        {
          if(nMode == MODE_RECORDING)
          {
            //  
            rc=waveInStop(hWaveIn);
            if(rc) wioInError(rc);
            nMode = MODE_STOP;
          }

          else if(nMode == MODE_PLAYING)
          {
            //   
            rc=waveOutPause(hWaveOut);
            if(rc) wioOutError(rc);
            nMode = MODE_PLAYINGPAUSED;
          }

          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }

        // -------------------------------------------------
        //  "Resume!"
        //    
        // -------------------------------------------------
        case CM_CTLRESUME:
        {
          if(nMode == MODE_PLAYINGPAUSED)
          {
            rc=waveOutRestart(hWaveOut);
            if(rc) wioOutError(rc);
            nMode = MODE_PLAYING;
            InvalidateRect(hwnd, NULL, TRUE);
          }
          return 0;
        }

        // -------------------------------------------------
        //  "Exit"  "File"
        //   
        // -------------------------------------------------
        case CM_FILEEXIT:
        {
          DestroyWindow(hwnd);
          return 0;
        }

        default:
          return 0;
      }
    }

// ------------------------------------------------------------
// MM_WOM_DONE
//   
// ------------------------------------------------------------
    case MM_WOM_DONE:
    {
      nMode = MODE_STOP;
      nPosition = 0;
      SetScrollPos(hScroll, SB_CTL, nPosition, TRUE);
      InvalidateRect(hwnd, NULL, TRUE);

      //     
      waveOutUnprepareHeader((HWAVEOUT)wParam,
       (LPWAVEHDR)lParam, sizeof(WAVEHDR));

      //     
      waveOutReset((HWAVEOUT)wParam);
      waveOutClose((HWAVEOUT)wParam);
      hWaveOut = 0;
      return 0;
    }

// ------------------------------------------------------------
// MM_WIM_DATA
//   
// ------------------------------------------------------------
    case MM_WIM_DATA:
    {
      nMode = MODE_STOP;
      nPosition = 0;
      SetScrollPos(hScroll, SB_CTL, nPosition, TRUE);
      InvalidateRect(hwnd, NULL, TRUE);

      //     
      waveInUnprepareHeader((HWAVEIN)wParam,
       (LPWAVEHDR)lParam, sizeof(WAVEHDR));

      //     
      wioFileSave("RECORDED.WAV");
      fNeedSave = FALSE;

      //  ,   
      GlobalFreePtr(waveiocbIn.lpWaveHdr);
      GlobalFreePtr(waveiocbIn.lpData);

      //     
      waveInReset((HWAVEIN)wParam);
      waveInClose((HWAVEIN)wParam);
      hWaveIn = 0;
      return 0;
    }

// ------------------------------------------------------------
// WM_TIMER
//   
// ------------------------------------------------------------
    case WM_TIMER:
    {
      if(nMode == MODE_RECORDING)
      {
        //      
        mmtimeIn.wType = TIME_SAMPLES;
        waveInGetPosition(hWaveIn,
          (LPMMTIME)&mmtimeIn, sizeof(MMTIME));

        //      
        nPosition =
          ((DWORD)SCROLL_SIZE * mmtimeIn.u.sample) / MAXSAMPLES;

        //    
        //    1  SCROLL_SIZE
        if(nPosition > SCROLL_SIZE) nPosition = SCROLL_SIZE;
        if(nPosition < 1) nPosition = 1;

        //    
        //       
        SetScrollPos(hScroll, SB_CTL, nPosition, TRUE);
      }

      else if(nMode == MODE_PLAYING)
      {
        //      
        mmtimeOut.wType = TIME_SAMPLES;
        waveOutGetPosition(hWaveOut,
        (LPMMTIME)&mmtimeOut, sizeof(MMTIME));

        //      
        nPosition =
          ((DWORD)SCROLL_SIZE * mmtimeOut.u.sample) /
           (waveiocbOut.dwDataSize / waveiocbOut.wBytesPerSample);

        //    
        //    1  SCROLL_SIZE
        if(nPosition > SCROLL_SIZE) nPosition = SCROLL_SIZE;
        if(nPosition < 1) nPosition = 1;

        //    
        //       
        SetScrollPos(hScroll, SB_CTL, nPosition, TRUE);
      }
      return 0;
    }

// ------------------------------------------------------------
// WM_DESTROY
//    
// ------------------------------------------------------------
    case WM_DESTROY:
    {
      //     
      KillTimer(hwnd, BEEP_TIMER);
      DestroyWindow(hScroll);

      //     , 
      //     
      if(nMode == MODE_RECORDING || nMode == MODE_RECORDINGPAUSED)
      {
        if(hWaveIn)
        {
          rc=waveInReset(hWaveIn);
            if(rc) wioInError(rc);
          rc=waveInClose(hWaveIn);
            if(rc) wioInError(rc);
        }
      }

      //   ,    , 
      //    
      if(fNeedSave)
      {
        GlobalFreePtr(waveiocbIn.lpWaveHdr);
        GlobalFreePtr(waveiocbIn.lpData);
      }

      else if(fFileLoaded)
      {
      //     , 
      //     
        if(nMode == MODE_PLAYING || nMode == MODE_PLAYINGPAUSED)
        {
          if(hWaveOut)
          {
            rc=waveOutReset(hWaveOut);
            if(rc) wioOutError(rc);

            rc=waveOutUnprepareHeader(hWaveOut, waveiocbOut.lpWaveHdr, sizeof(WAVEHDR));
            if(rc) wioOutError(rc);

            rc=waveOutClose(hWaveOut);
            if(rc) wioOutError(rc);
          }
        }

        nMode = MODE_STOP;

        //  
        GlobalFreePtr(waveiocbOut.lpWaveHdr);
        GlobalFreePtr(waveiocbOut.lpData);
        GlobalFreePtr(waveiocbOut.lpFmt);
      }
      PostQuitMessage(0);
      return 0;
    }
    default:
      break;
  }
  return DefWindowProc(hwnd, msg, wParam, lParam);
}

