// ----------------------------------------
//    WinHelp
// ----------------------------------------
#define STRICT
#include <windows.h>
#include <mem.h>
#include "helpmwh.hpp"

BOOL InitApp(HINSTANCE);
LRESULT CALLBACK _export WndProc(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK    _export DlgProc(HWND, UINT, WPARAM, LPARAM);

char const szClassName[]   = "HelpmWhClass";
char const szWindowTitle[] = "WinHelp Demo";
HACCEL     hAccel;
BOOL       bF1 = FALSE;
HWND       hwndDlg = NULL;
DLGPROC    lpfnDlgProc;
HINSTANCE  hInst;

// =====================================
//  WinMain
// =====================================
#pragma argsused
int PASCAL
WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
  LPSTR lpszCmdLine, int nCmdShow)
{
  MSG  msg;   //     
  HWND hwnd;  //    

  if(!InitApp(hInstance))
      return FALSE;

  hInst = hInstance;

  hwnd = CreateWindow(
    szClassName, szWindowTitle, WS_OVERLAPPEDWINDOW,
    CW_USEDEFAULT, CW_USEDEFAULT,
    CW_USEDEFAULT,  CW_USEDEFAULT,
    0, 0, hInstance, NULL);

  if(!hwnd)
    return FALSE;

  ShowWindow(hwnd, nCmdShow);
  UpdateWindow(hwnd);

  //   
  hAccel = LoadAccelerators(hInstance, "APP_ACCEL");

  while(GetMessage(&msg, 0, 0, 0))
  {
    if(!TranslateAccelerator(hwnd, hAccel, &msg))
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }
  }
  return msg.wParam;
}

// =====================================
//  InitApp
//    
// =====================================
BOOL InitApp(HINSTANCE hInstance)
{
  ATOM aWndClass;
  WNDCLASS wc;

  memset(&wc, 0, sizeof(wc));
  wc.lpszMenuName  = "APP_MENU";
  wc.style         = 0;
  wc.lpfnWndProc   = (WNDPROC) WndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
  wc.lpszClassName = (LPSTR)szClassName;

  aWndClass = RegisterClass(&wc);
  return (aWndClass != 0);
}

// =====================================
//  WndProc
// =====================================
LRESULT CALLBACK _export
WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  switch (msg)
  {
    //   ,    
    //      
    case WM_ENTERIDLE:
    {
      //    
      // ,     <F1> 
      if((wParam == MSGF_MENU) &&
         ((GetKeyState(VK_F1) & 0x8000) != 0))
      {
        //     <F1>, 
        //  bF1,    
        //  <Enter>
        bF1 = TRUE;
        PostMessage(hwnd, WM_KEYDOWN, VK_RETURN, 0l);
      }

      //   <F1>     
      //  ,   
      //  
      else if((wParam == MSGF_DIALOGBOX) &&
         ((GetKeyState(VK_F1) & 0x8000) != 0))
      {
        hwndDlg = (HWND)LOWORD(lParam);
        WinHelp(hwnd, "hlpmore.hlp", HELP_CONTENTS, 0L);
      }
      break;
    }

    case WM_COMMAND:
    {
      switch (wParam)
      {
        //     
        case CM_HELPINDEX:
        {
          WinHelp(hwnd, "hlpmore.hlp", HELP_CONTENTS, 0L);
          return 0L;
        }
        case CM_HELPCOMMANDS:
        {
          //       <F1>,
          //   ,  
          //   
          if(bF1)
          {
            WinHelp(hwnd, "hlpmore.hlp", HELP_CONTENTS, 0L);
            bF1 = FALSE;
          }
          else
            WinHelp(hwnd, "hlpmore.hlp", HELP_CONTEXT,
              (DWORD)IDN_cmd);
          return 0L;
        }

        case CM_HELPKEYBOARD:
        {
          if(bF1)
          {
            WinHelp(hwnd, "hlpmore.hlp", HELP_CONTENTS, 0L);
            bF1 = FALSE;
          }
          else
            WinHelp(hwnd, "hlpmore.hlp", HELP_CONTEXT,
              (DWORD)IDN_key);
          return 0L;
        }

        case CM_HELPUSING_HELP:
        {
          if(bF1)
          {
            WinHelp(hwnd, "hlpmore.hlp", HELP_CONTENTS, 0L);
            bF1 = FALSE;
          }
          else
            WinHelp(hwnd, "hlpmore.hlp", HELP_HELPONHELP, 0L);
          return 0L;
        }

        case CM_HELPABOUT:
        {
          if(bF1)
          {
            WinHelp(hwnd, "hlpmore.hlp", HELP_CONTENTS, 0L);
            bF1 = FALSE;
          }
          else
          {
            lpfnDlgProc =
              (DLGPROC)MakeProcInstance((FARPROC)DlgProc, hInst);
            DialogBox(hInst, "ABOUT_DIALOG", hwnd, lpfnDlgProc);
          }
          return 0;
        }

        case CM_FILEPRINTER_SETUP:
        case CM_FILEPAGE_SETUP:
        case CM_FILEPRINT:
        case CM_FILESAVEAS:
        case CM_FILESAVE:
        case CM_FILEOPEN:
        case CM_FILENEW:
        {
          if(bF1)
          {
            WinHelp(hwnd,"hlpmore.hlp", HELP_CONTEXT,
              (DWORD)IDN_file);
            bF1 = FALSE;
          }
          else
	         MessageBox(hwnd, " File",
	            "WinHelp Demo", MB_OK);
          return 0;
        }

        case CM_EDITPASTE:
        case CM_EDITCOPY:
        case CM_EDITCUT:
        case CM_EDITUNDO:
        {
          if(bF1)
          {
            WinHelp(hwnd,"hlpmore.hlp", HELP_CONTEXT,
              (DWORD)IDN_edit);
            bF1 = FALSE;
          }
          else
	         MessageBox(hwnd, " Edit",
	            "WinHelp Demo", MB_OK);
          return 0;
        }

        case CM_VIEWNORMAL:
        case CM_VIEWZOOM:
        {
          if(bF1)
          {
            WinHelp(hwnd,"hlpmore.hlp", HELP_CONTEXT,
              (DWORD)IDN_view);
            bF1 = FALSE;
          }
          else
	         MessageBox(hwnd, " View",
	            "WinHelp Demo", MB_OK);
          return 0;
        }

        case CM_FILEEXIT:
        {
          DestroyWindow(hwnd);
          return 0;
        }

        default:
          return 0;
      }
    }

    case WM_DESTROY:
    {
      PostQuitMessage(0);
      return 0;
    }

    default:
      break;
  }
  return DefWindowProc(hwnd, msg, wParam, lParam);
}

// =====================================
//  DldProc
// =====================================
#pragma argsused
BOOL CALLBACK _export
DlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
  switch(msg)
  {
    case WM_INITDIALOG:
    {
      return TRUE;
    }

    case WM_COMMAND:
    {
      switch(wParam)
      {
        case IDOK:
        case IDCANCEL:
        {
          EndDialog(hdlg, 0);
          return TRUE;
        }
      }
    }
  }
  return FALSE;
}
