// ==================================================================
// MDI-   Toolbar  Statusbar
// ==================================================================

#define STRICT
#include <windows.h>
#include <mem.h>
#include "mditb.hpp"

//  
BOOL InitApp(HINSTANCE);
LRESULT CALLBACK _export FrameWndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK _export ChildWndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK _export TbWndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK _export SbWndProc(HWND, UINT, WPARAM, LPARAM);

//   
char const szFrameClassName[] = "MDITBAppClass";
char const szChildClassName[] = "MDITBChildAppClass";
char const szTbClassName[] =    "MDITBCtrlAppClass";
char const szSbClassName[] =    "MDISBCtrlAppClass";

//  
char const szWindowTitle[] = "MDI Application";

HINSTANCE hInst;

HWND hwndFrame;  //  Frame Window
HWND hwndClient; //  Client Window
HWND hwndChild;  //  Child Window
HWND hwndTb;     //  Toolbar
HWND hwndSb;     //  Statusbar

// =====================================
//  WinMain
// =====================================
#pragma argsused

int PASCAL
WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
	LPSTR lpszCmdLine, int nCmdShow)
{
  MSG  msg;          //     

  hInst = hInstance; //   

  if(hPrevInstance)  //    
    return FALSE;    //    

  //  
  if(!InitApp(hInstance))
      return FALSE;

  //     - Frame Window
  hwndFrame = CreateWindow(
    szFrameClassName,    //   
    szWindowTitle,       //  
    WS_OVERLAPPEDWINDOW, //  
    CW_USEDEFAULT, 0,    //    
    CW_USEDEFAULT, 0,    // ,   
    0,                   //   
    0,                   //  
    hInstance,           //  
    NULL);               //    

  //     ,  
  if(!hwndFrame)
    return FALSE;

  //   
  ShowWindow(hwndFrame, nCmdShow);
  UpdateWindow(hwndFrame);

  //    
  while(GetMessage(&msg, NULL, 0, 0))
  {
    //   MDI-
    if(!TranslateMDISysAccel(hwndClient, &msg))
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }
  }
  return msg.wParam;
}

// =====================================
//  InitApp
//    
// =====================================

BOOL
InitApp(HINSTANCE hInstance)
{
  ATOM aWndClass; //    
  WNDCLASS wc;    //   
                  //  

  //      
  // (. .   Frame Window)
  memset(&wc, 0, sizeof(wc));
  wc.lpszMenuName  = "APP_MENU";
  wc.style         = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc   = (WNDPROC)FrameWndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon(hInstance, "APP_ICON");
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)(COLOR_APPWORKSPACE + 1);
  wc.lpszClassName = (LPSTR)szFrameClassName;
  aWndClass = RegisterClass(&wc);

  if(!aWndClass)
    return FALSE;

  //       Document Window
  memset(&wc, 0, sizeof(wc));
  wc.lpszMenuName  = 0;
  wc.style         = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc   = (WNDPROC)ChildWndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon(hInstance, "APPCLIENT_ICON");
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
  wc.lpszClassName = (LPSTR)szChildClassName;
  aWndClass = RegisterClass(&wc);

  if(!aWndClass)
    return FALSE;

  //     Toolbar
  memset(&wc, 0, sizeof(wc));
  wc.lpszMenuName  = 0;
  wc.style         = 0;
  wc.lpfnWndProc   = (WNDPROC)TbWndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = NULL;
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
  wc.lpszClassName = (LPSTR)szTbClassName;
  aWndClass = RegisterClass(&wc);

  if(!aWndClass)
    return FALSE;

  //     Statusbar
  memset(&wc, 0, sizeof(wc));
  wc.lpszMenuName  = 0;
  wc.style         = 0;
  wc.lpfnWndProc   = (WNDPROC)SbWndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = NULL;
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
  wc.lpszClassName = (LPSTR)szSbClassName;
  aWndClass = RegisterClass(&wc);

  if(!aWndClass)
    return FALSE;

  return TRUE;
}

// =====================================
//  FrameWndProc
// =====================================

LRESULT CALLBACK _export
FrameWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  //     Client Window
  CLIENTCREATESTRUCT clcs;

  //      Document Window
  MDICREATESTRUCT    mdics;

  switch (msg)
  {
    //    Toolbar, Statusbar.
    //    Client Window
    case WM_SIZE:
    {
      //   Toolbar   
      //  Frame Window
      MoveWindow(hwndTb,
        0,                      // x-
        0,                      // y-
        LOWORD(lParam),         // 
        TBAR_SIZE,              // 
        TRUE);                  //   

      //   Statusbar   
      //  Frame Window
      MoveWindow(hwndSb,
        0,                      // x-
        HIWORD(lParam) - SBAR_SIZE,  // y-
        LOWORD(lParam),         // 
        SBAR_SIZE,              // 
        TRUE);                  //   

      //        
      //    , 
      //  MoveWindow
      if(wParam != SIZEICONIC && hwndClient)
      {
        MoveWindow(hwndClient,
          0,                      // x-
          TBAR_SIZE,              // y-
          LOWORD(lParam),         // 
          HIWORD(lParam) - (TBAR_SIZE + SBAR_SIZE), // 
          TRUE);                  //   

        //     
        //   WM_SIZE  DefFrameProc,
        //      
        return 0;
      }
      break;
    }


    //    Frame Window 
    //  Client Window,    
    //   Document Window
    case WM_CREATE:
    {
      //      clcs 
      //   "Window".      ,
      //    1 ( "File"   0) 
      clcs.hWindowMenu = GetSubMenu(GetMenu(hwnd), 1);

      //     Document Window
      clcs.idFirstChild = 500;
      
      //   Client Window 
      hwndClient = CreateWindow(
        "MDICLIENT",    //   
        NULL,           //  
        WS_CHILD | WS_CLIPCHILDREN | WS_VISIBLE | //  
          WS_HSCROLL | WS_VSCROLL,  
        0, 0, 0, 0,
        hwnd,           //   
        (HMENU)1,       //  
        hInst,          //  
        (LPSTR)&clcs);  //    

      //   Toolbar
      hwndTb = CreateWindow(
        szTbClassName,  //   
        NULL,           //  
        WS_CHILD | WS_CLIPCHILDREN | WS_VISIBLE | WS_DLGFRAME, //  
        0, 0, 0, 0,
        hwnd,           //   
        (HMENU)2,       //  
        hInst,          //  
        NULL);  //    

      //   Statusbar
      hwndSb = CreateWindow(
        szSbClassName,  //   
        NULL,           //  
        WS_CHILD | WS_CLIPCHILDREN | WS_VISIBLE | WS_DLGFRAME, //  
        0, 0, 0, 0,
        hwnd,           //   
        (HMENU)3,       //  
        hInst,          //  
        NULL);  //    

      break;
    }

    //      
    case WM_COMMAND:
    {
      switch (wParam)
      {
        //    Document Window
        case CM_FILENEW:
        {
          //   MDICREATESTRUCT
          mdics.szClass = szChildClassName;   //  
          mdics.szTitle = "MDI Child Window"; //  
          mdics.hOwner  = hInst;              //   
          mdics.x       = CW_USEDEFAULT;      //   Document Window
          mdics.y       = CW_USEDEFAULT;
          mdics.cx      = CW_USEDEFAULT;
          mdics.cy      = CW_USEDEFAULT;
          mdics.style   = 0;                  //  
          mdics.lParam  = NULL;               // 32- 

          //   WM_MDICREATE  Client Window.
          //       Document Window
          hwndChild = (HWND)SendMessage(hwndClient, WM_MDICREATE, 0, (LPARAM)&mdics);

          break;
        }

        //   Document Window    
        case CM_WINDOWTILE:
        {
          SendMessage(hwndClient, WM_MDITILE, 0, NULL);
          break;
        }

        //   Document Window  
        case CM_WINDOWCASCADE:
        {
          SendMessage(hwndClient, WM_MDICASCADE, 0, NULL);
          break;
        }

        //     Document Window
        //     Client Window
        case CM_WINDOWICONS:
        {
          SendMessage(hwndClient, WM_MDIICONARRANGE, 0, NULL);
          break;
        }

        //    Document Window
        case CM_WINDOWCLOSEALL:
        {
          HWND hwndTemp;

          //   Client Window    
          //    Document Window    
          ShowWindow(hwndClient, SW_HIDE);

          for(;;)
          {
            //       Client Window
            hwndTemp = GetWindow(hwndClient, GW_CHILD);
            //     ,   
            if(!hwndTemp)
              break;

            //  -
            while(hwndTemp && GetWindow(hwndTemp, GW_OWNER))
              hwndTemp = GetWindow(hwndTemp, GW_HWNDNEXT);

            //    Document Window
            if(hwndTemp)
              SendMessage(hwndClient, WM_MDIDESTROY, (WPARAM)hwndTemp, NULL);
            else
              break;
          }

          //   Client Window
          ShowWindow(hwndClient, SW_SHOW);
          break;
        }

        case CM_HELPABOUT:
        {
	       MessageBox(hwnd,
            " MDIAPP\n(C)  .., 1995",
	         "Simple MDI Application", MB_OK | MB_ICONINFORMATION);
          break;
        }

        //   
        case CM_FILEEXIT:
        {
          DestroyWindow(hwnd);
          break;
        }

        default:
          break;
      }

      //     Document Window
      HWND hwndChild =
        (HWND)LOWORD(SendMessage(hwndClient, WM_MDIGETACTIVE, 0, 0l));

      //   ,    WM_COMMAND
      if(IsWindow(hwndChild))
        SendMessage(hwndChild, WM_COMMAND, wParam, lParam);

      return DefFrameProc(hwnd, hwndClient, msg, wParam, lParam);
    }

    case WM_DESTROY:
    {
      PostQuitMessage(0);
      break;
    }

    default:
      break;
  }
  return DefFrameProc(hwnd, hwndClient, msg, wParam, lParam);
}

// =====================================
//  ChildWndProc
// =====================================

LRESULT CALLBACK _export
ChildWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  HDC hdc;
  PAINTSTRUCT ps;
  RECT rc;

  switch (msg)
  {
    case WM_PAINT:
    {
      hdc = BeginPaint(hwnd, &ps);
      GetClientRect(hwnd, &rc);

      DrawText(hdc, "Child Window", -1, &rc,
        DT_SINGLELINE | DT_CENTER | DT_VCENTER);

      EndPaint(hwnd, &ps);
    }

    default:
      break;
  }
  return DefMDIChildProc(hwnd, msg, wParam, lParam);
}

// =====================================
//  TbWndProc
// =====================================

LRESULT CALLBACK _export
TbWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  HDC hdc;
  PAINTSTRUCT ps;
  RECT rc;

  switch (msg)
  {
    case WM_PAINT:
    {
      hdc = BeginPaint(hwnd, &ps);
      GetClientRect(hwnd, &rc);

      DrawText(hdc, "Toolbar Window", -1, &rc,
        DT_SINGLELINE | DT_CENTER | DT_VCENTER);

      EndPaint(hwnd, &ps);
    }

    default:
      break;
  }
  return DefMDIChildProc(hwnd, msg, wParam, lParam);
}

// =====================================
//  SbWndProc
// =====================================

LRESULT CALLBACK _export
SbWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  HDC hdc;
  PAINTSTRUCT ps;
  RECT rc;

  switch (msg)
  {
    case WM_PAINT:
    {
      hdc = BeginPaint(hwnd, &ps);
      GetClientRect(hwnd, &rc);

      DrawText(hdc, "Statusbar Window", -1, &rc,
        DT_SINGLELINE | DT_CENTER | DT_VCENTER);

      EndPaint(hwnd, &ps);
    }

    default:
      break;
  }
  return DefMDIChildProc(hwnd, msg, wParam, lParam);
}
