#define STRICT
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include "resource.h"
#include "afxres.h"
#include "cdplay.h"
#include <mmsystem.h>

DWORD dwrc;

MCI_OPEN_PARMS   MCIOpen;
MCI_SET_PARMS    MCISet;
MCI_STATUS_PARMS MCIStatus;
MCI_PLAY_PARMS   MCIPlay;

BOOL bMediaPresent = FALSE;
BOOL bPaused       = FALSE;
UINT nMode = 0;
extern UINT nCurTrack;
extern UINT nTrackCnt;
extern HWND hTrackBar;

HWND hwndCurTrack = NULL;

// -----------------------------------------------------
//  CdInit
// -----------------------------------------------------
BOOL CdInit(void)
{
  //    -
  MCIOpen.lpstrDeviceType = (LPSTR)MCI_DEVTYPE_CD_AUDIO;
  dwrc = mciSendCommand(0, MCI_OPEN,
    MCI_OPEN_TYPE | MCI_OPEN_TYPE_ID, (DWORD)&MCIOpen);
  if(dwrc)
  {
    mciwioError(dwrc);
    return FALSE;
  }

  //   
  MCISet.dwTimeFormat = MCI_FORMAT_TMSF;
  dwrc = mciSendCommand(MCIOpen.wDeviceID, MCI_SET,
    MCI_SET_TIME_FORMAT, (DWORD)&MCISet);
  if(dwrc)
  {
    mciwioError(dwrc);
    return FALSE;
  }
  return TRUE;
}

// -----------------------------------------------------
//  CdClose
// -----------------------------------------------------
void CdClose(void)
{
  //    -
  dwrc = mciSendCommand(MCIOpen.wDeviceID, MCI_CLOSE, 0, 0);
  if(dwrc)
    mciwioError(dwrc);
}

//-----------------------------------------------------
// mciwioError
//  
//-----------------------------------------------------
void mciwioError(DWORD dwrc)
{
  BYTE szBuf[MAXERRORLENGTH];

  if(mciGetErrorString(dwrc, (LPSTR)szBuf, MAXERRORLENGTH))
    MessageBox(NULL, szBuf,
      "MCIWAVE Error", MB_ICONEXCLAMATION);
  else
    MessageBox(NULL, " ",
      "MCIWAVE Error", MB_ICONEXCLAMATION);
}

// -----------------------------------------------------
//  CdUpdateState
// -----------------------------------------------------
BOOL CdUpdateState(HWND hdlg)
{
  BOOL fNeedUpdate = FALSE;
  UINT nCurMode;

  //     CD
  MCIStatus.dwItem = MCI_STATUS_MODE;
  mciSendCommand(MCIOpen.wDeviceID, MCI_STATUS,
    MCI_STATUS_ITEM | MCI_WAIT, (DWORD)&MCIStatus);

  // ,       
  if((MCIStatus.dwReturn == MCI_MODE_NOT_READY) ||
     (MCIStatus.dwReturn == MCI_MODE_OPEN))
  {
    //   
    nCurMode = CD_EMPTY;
  }
  else if((MCIStatus.dwReturn == MCI_MODE_STOP) &&
    bPaused)
  {
    //   
    nCurMode = CD_PAUSED;
  }
  else if(MCIStatus.dwReturn == MCI_MODE_PLAY)
  {  
    //     
    nCurMode = CD_PLAYING;
  }
  else
  {
    //  
    nCurMode = CD_READY;
  }

  //      
  //  ,    
  if(nMode != nCurMode)
  {
    fNeedUpdate = TRUE;
    nMode = nCurMode;
    
    //      ,
    //  .   
    //      
    if(nCurMode == CD_PLAYING)
      Animate_Play(GetDlgItem(hdlg, IDC_ANIMATE), 0, -1, -1);
    else
      Animate_Stop(GetDlgItem(hdlg, IDC_ANIMATE));
  }

  // ,   -
  MCIStatus.dwItem = MCI_STATUS_MEDIA_PRESENT;
    mciSendCommand(MCIOpen.wDeviceID, MCI_STATUS,
    MCI_STATUS_ITEM | MCI_WAIT, (DWORD)&MCIStatus);

  //  - , 
  //   
  if((!bMediaPresent) && MCIStatus.dwReturn)
  {
    bMediaPresent = TRUE;
    bPaused = FALSE;
    nCurTrack = 0;

    MCIStatus.dwItem = MCI_STATUS_NUMBER_OF_TRACKS;
    mciSendCommand(MCIOpen.wDeviceID, MCI_STATUS,
      MCI_STATUS_ITEM | MCI_WAIT, (DWORD)&MCIStatus);

    nTrackCnt = MCIStatus.dwReturn;

    //    
    //    Trackbar
    SendMessage(hTrackBar, TBM_SETRANGE, TRUE, 
      MAKELPARAM(1, nTrackCnt));
  }

  //  -  , 
  //       
  else if((bMediaPresent) && !MCIStatus.dwReturn)
  {
    bMediaPresent = FALSE;
    bPaused = FALSE;
  }

  //      ,
  //    
  if(nCurMode == CD_PLAYING)
  {
    //   
    MCIStatus.dwItem = MCI_STATUS_POSITION;
    mciSendCommand(MCIOpen.wDeviceID, MCI_STATUS,
      MCI_STATUS_ITEM | MCI_WAIT, (DWORD)&MCIStatus);

    //    ,  
    //       
    if(nCurTrack != (UINT)MCI_TMSF_TRACK(MCIStatus.dwReturn))
    {
      nCurTrack = (UINT)MCI_TMSF_TRACK(MCIStatus.dwReturn);
      fNeedUpdate = TRUE;
    }
  }
  return fNeedUpdate;
}

//-----------------------------------------------------
// CdPlay
//   
//-----------------------------------------------------
void CdPlay(HWND hwnd, UINT nTrack)
{
  if(bMediaPresent)
  {
    bPaused = FALSE;

    MCIPlay.dwCallback = (DWORD)hwnd;
    MCIPlay.dwFrom = MCI_MAKE_TMSF(nTrack, 0, 0, 0);

    dwrc = mciSendCommand(MCIOpen.wDeviceID, MCI_PLAY,
      MCI_FROM | MCI_NOTIFY, (DWORD)&MCIPlay);
    if(dwrc)
      mciwioError(dwrc);
  }
}

//-----------------------------------------------------
// CdStop
//   
//-----------------------------------------------------
void CdStop(void)
{
  if(bMediaPresent)
  {
    bPaused = FALSE;
    nCurTrack = 0;
    mciSendCommand(MCIOpen.wDeviceID, MCI_STOP, 0, 0);
  }
}

//-----------------------------------------------------
// CdPause
//    
//-----------------------------------------------------
void CdPause(void)
{
  if(bMediaPresent)
  {
    if(!bPaused)
    {
      bPaused = TRUE;
      mciSendCommand(MCIOpen.wDeviceID, MCI_PAUSE, 0, 0);
    }
  }
}

//-----------------------------------------------------
// CdResume
//     
//-----------------------------------------------------
void CdResume(HWND hwnd)
{
  if(bMediaPresent)
  {
    if(bPaused)
    {
      bPaused = FALSE;
      MCIPlay.dwCallback = (DWORD)hwnd;
      mciSendCommand(MCIOpen.wDeviceID, MCI_PLAY,
        MCI_NOTIFY, (DWORD)&MCIPlay);
    }
  }
}

//-----------------------------------------------------
// CdPlayNext
//   
//-----------------------------------------------------
void CdPlayNext(HWND hwnd)
{
  UINT nNewTrack;
  
  if(bMediaPresent)
  {
    //    - ,
    //     .
    //   -   
    if(nCurTrack == nTrackCnt)
      nNewTrack = 1;
    else
      nNewTrack = nCurTrack + 1;

    CdPlay(hwnd, nNewTrack);
  }
}

//-----------------------------------------------------
// CdPlayPrev
//   
//-----------------------------------------------------
void CdPlayPrev(HWND hwnd)
{
  UINT nNewTrack;
  if(bMediaPresent)
  {
    //    - ,
    //   
    if(nCurTrack <= 1)
      nNewTrack = nTrackCnt;
    else
      nNewTrack = nCurTrack - 1;

    CdPlay(hwnd, nNewTrack);
  }
}

//-----------------------------------------------------
// CdEject 
//  -
//-----------------------------------------------------
void CdEject(void)
{
  mciSendCommand(MCIOpen.wDeviceID, MCI_SET,
    MCI_SET_DOOR_OPEN, 0);
}
