// ----------------------------------------
//  Clipboard
// ----------------------------------------
#define STRICT
#include <windows.h>
#include <mem.h>
#include "clipshow.hpp"

BOOL InitApp(HINSTANCE);
LRESULT CALLBACK _export WndProc(HWND, UINT, WPARAM, LPARAM);
void DrawBitmap(HDC, int, int, HBITMAP);
void PrepareMetaFile(HDC, LPMETAFILEPICT, int, int);
void SizeToHiMetric(int*, int*);
void HiMetricToSize(int*, int*);
void HiMetricToSizeScaled(int*, int*, int, int);

char const szClassName[]   = "ClipShowClass";
char const szWindowTitle[] = "Clipboard Show";
// =====================================
//  WinMain
// =====================================
#pragma argsused
int PASCAL
WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
        LPSTR lpszCmdLine, int nCmdShow)
{
  MSG  msg;   //     
  HWND hwnd;  //    

//      
  if(hPrevInstance)
    return FALSE;

  if(!InitApp(hInstance))
    return FALSE;

  hwnd = CreateWindow(
    szClassName, szWindowTitle,
    WS_OVERLAPPEDWINDOW,
    CW_USEDEFAULT, CW_USEDEFAULT,
    CW_USEDEFAULT, CW_USEDEFAULT,
    0, 0, hInstance, NULL);
  if(!hwnd)
    return FALSE;

  ShowWindow(hwnd, nCmdShow);
  UpdateWindow(hwnd);

  while(GetMessage(&msg, 0, 0, 0))
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }
  return msg.wParam;
}
// =====================================
//  InitApp
//    
// =====================================
BOOL
InitApp(HINSTANCE hInstance)
{
  ATOM aWndClass; //    
  WNDCLASS wc;    //   
                  //  
  memset(&wc, 0, sizeof(wc));
  wc.lpszMenuName  = "APP_MENU";
  wc.style         = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc   = (WNDPROC) WndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon(hInstance, "APP_ICON");
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
  wc.lpszClassName = (LPSTR)szClassName;

  aWndClass = RegisterClass(&wc);
  return (aWndClass != 0);
}
// =====================================
//  WndProc
// =====================================
LRESULT CALLBACK _export
WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  HDC hdc;
  PAINTSTRUCT ps;
  RECT rc;
  static HWND hwndNextViewer;
  HGLOBAL hglbClipData;
  LPSTR   lpszClipBuf;
  HBITMAP hBitmap;
  static HPALETTE hPal, hOldPal;
  static int cxClient, cyClient;
  HMETAFILE hmf;
  LPMETAFILEPICT lpmfp;

  switch (msg)
  {
    case WM_CREATE:
    {
      //      Clipboard
      hwndNextViewer = SetClipboardViewer(hwnd);
      return 0;
    }
    case WM_SIZE:
    {
      cxClient = LOWORD(lParam);
      cyClient = HIWORD(lParam);
      return 0;
    }
    case WM_PAINT:
    {
      hdc = BeginPaint(hwnd, &ps);
      GetClientRect(hwnd, &rc);

      //  Clipboard
      OpenClipboard(hwnd);

      //     
      hmf = (HMETAFILE)GetClipboardData(CF_METAFILEPICT);
      if(hmf != NULL)
      {
        //    
        lpmfp = (LPMETAFILEPICT)GlobalLock(hmf);
        if(lpmfp != NULL)
        {
          //   
          SaveDC(hdc);

          //    
          //      
          PrepareMetaFile(hdc, lpmfp, cxClient, cyClient);

          //   
          PlayMetaFile(hdc, lpmfp->hMF);

          //   
          RestoreDC(hdc, -1);

          //    
          GlobalUnlock(hmf);
        }
      }
      else
      {
        //     CF_BITMAP
        hBitmap = (HBITMAP)GetClipboardData(CF_BITMAP);
        if(hBitmap != NULL)
        {
          //  
          hPal = (HPALETTE)GetClipboardData(CF_PALETTE);
          if(hPal)
          {
            //  Clipboard  ,
            //    
            hOldPal = SelectPalette(hdc, hPal, FALSE);
            RealizePalette(hdc);
          }
          //    
          DrawBitmap(hdc, 0, 0, hBitmap);
        }
        //   
        else
        {
          hglbClipData = GetClipboardData(CF_TEXT);
          if(hglbClipData)
          {
            //      
            lpszClipBuf = (LPSTR)GlobalLock(hglbClipData);
            DrawText(hdc, lpszClipBuf, -1, &rc, DT_EXPANDTABS);

            //   
            GlobalUnlock(hglbClipData);
          }
        }
      }
      //  Clipboard
      CloseClipboard();
      EndPaint(hwnd, &ps);
    }
    //      
    // Clipboard
    case WM_CHANGECBCHAIN:
    {
      //     
      //      ,
      //    
      //  
      if(wParam == (WPARAM)hwndNextViewer)
        hwndNextViewer = (HWND)LOWORD(lParam);

      //     
      //  
      else if(hwndNextViewer)
        SendMessage(hwndNextViewer, msg, wParam, lParam);
      return 0;
        
    }
    //  Clipboard ,  
    //    
    case WM_DRAWCLIPBOARD:
    {
      //       ,
      //     
      if(hwndNextViewer)
        SendMessage(hwndNextViewer, msg, wParam, lParam);
      InvalidateRect(hwnd, NULL, TRUE);
      return 0;
    }
    case WM_COMMAND:
    {
      switch (wParam)
      {
        case CM_HELPABOUT:
        {
	       MessageBox(hwnd,
            " CLIPSHOW\n(C)  .., 1995",
	         (LPSTR)szWindowTitle, MB_OK | MB_ICONINFORMATION);
          return 0;
        }
        case CM_FILEEXIT:
        {
          DestroyWindow(hwnd);
          return 0;
        }
        default:
          return 0;
      }
    }
    //   
    case WM_PALETTECHANGED:
    {
      if(hwnd == (HWND) wParam)
        break;
    }
    case WM_QUERYNEWPALETTE:
    {
      int nChanged;

      hdc = GetDC(hwnd);
      hOldPal = SelectPalette(hdc, hPal,
        (msg == WM_QUERYNEWPALETTE) ? FALSE : TRUE);
      nChanged = RealizePalette(hdc);
      SelectPalette(hdc, hOldPal, TRUE);
      ReleaseDC(hwnd, hdc);
      if(nChanged)
        InvalidateRect(hwnd, NULL, TRUE);
      return nChanged;
    }
    //     
    //    Clipboard,
    //      
    case WM_DESTROY:
    {
      ChangeClipboardChain(hwnd, hwndNextViewer);
      PostQuitMessage(0);
      return 0;
    }
    default:
      break;
  }
  return DefWindowProc(hwnd, msg, wParam, lParam);
}
// =====================================
//  DrawBitmap
// =====================================
void
DrawBitmap(HDC hDC, int x, int y, HBITMAP hBitmap)
{
  HBITMAP hbm, hOldbm;
  HDC hMemDC;
  BITMAP bm;
  POINT  ptSize, ptOrg;

  hMemDC = CreateCompatibleDC(hDC);
  hOldbm = (HBITMAP)SelectObject(hMemDC, hBitmap);
  if (hOldbm)
  {
    SetMapMode(hMemDC, GetMapMode(hDC));
    GetObject(hBitmap, sizeof(BITMAP), (LPSTR) &bm);

    ptSize.x = bm.bmWidth;   // 
    ptSize.y = bm.bmHeight;  // 
    DPtoLP(hDC, &ptSize, 1);

    ptOrg.x = 0;
    ptOrg.y = 0;
    DPtoLP(hMemDC, &ptOrg, 1);

    BitBlt(hDC, x, y, ptSize.x, ptSize.y,
	   hMemDC, ptOrg.x, ptOrg.y, SRCCOPY);
    SelectObject(hMemDC, hOldbm);
  }
  DeleteDC(hMemDC);
}
// =====================================
//  PrepareMetaFile
// =====================================
void
PrepareMetaFile(HDC hdc, LPMETAFILEPICT lpmfp,
  int cxClient, int cyClient)
{
  int x, y;

  //    ,
  //     
  SetMapMode(hdc, lpmfp->mm);

  //     
  //   xExt  yExt 
  if(lpmfp->mm == MM_ISOTROPIC ||
     lpmfp->mm == MM_ANISOTROPIC)
  {
    //  xExt  , 
    //    
    //   
    if(lpmfp->xExt == 0)
      SetViewportExtEx(hdc, cxClient, cyClient, NULL);

    //  xExt  , 
    //      ,
    //    
    else if(lpmfp->xExt > 0)
    {
      x = lpmfp->xExt;
      y = lpmfp->yExt;

      //   
      //     
      HiMetricToSize(&x, &y);
      SetViewportExtEx(hdc, x, y, NULL);
    }

    //  xExt  ,  
    // ,   
    else if(lpmfp->xExt < 0)
    {
      x = -lpmfp->xExt;
      y = -lpmfp->yExt;

      //   
      //       
      //  
      HiMetricToSizeScaled(&x, &y, cxClient, cyClient);
      SetViewportExtEx(hdc, x, y, NULL);
    }
  }
  //     
  //    ,
  //    
  else
    SetViewportExtEx(hdc, lpmfp->xExt, lpmfp->yExt, NULL);
}
// =====================================
//  SizeToHiMetric
// =====================================
void SizeToHiMetric(int *width, int *height)
{
  HDC hDC = GetDC(0);

  //     
  //      
  int dpiX = GetDeviceCaps(hDC, LOGPIXELSX);
  int dpiY = GetDeviceCaps(hDC, LOGPIXELSY);

  //   
  const long HiMetricPerInch = 2540;

  //  
  if(width)  *width  = int(*width * HiMetricPerInch / dpiX);
  if(height) *height = int(*height * HiMetricPerInch / dpiY);

  ReleaseDC(0, hDC);
}
// =====================================
//  HiMetricToSize
// =====================================
void HiMetricToSize(int *width, int *height)
{
  HDC hDC = GetDC(0);
  long int dpiX = GetDeviceCaps(hDC, LOGPIXELSX);
  long int dpiY = GetDeviceCaps(hDC, LOGPIXELSY);
  const long HiMetricPerInch = 2540;

  if(width  != NULL) *width  =
    int(*width * dpiX / HiMetricPerInch);
  if(height != NULL) *height =
    int(*height * dpiY / HiMetricPerInch);

  ReleaseDC (0, hDC);
}
// =====================================
//  HiMetricToSizeScaled
// =====================================
void HiMetricToSizeScaled (int *width, int *height,
  int cxClient, int cyClient)
{
  HDC hDC = GetDC(0);
  long int dpiX = GetDeviceCaps(hDC, LOGPIXELSX); 
  long int dpiY = GetDeviceCaps(hDC, LOGPIXELSY);
  const long HiMetricPerInch = 2540;
  int xPic, yPic;

  //    
  if (width !=  NULL) xPic =
    int(*width * dpiX / HiMetricPerInch);
  if (height != NULL) yPic =
    int(*height * dpiY / HiMetricPerInch);

  //   
  //    
  if((cxClient*100L) / xPic > (cyClient*100L) / yPic)
  {
    //    
    //  , 
    //  
    xPic = ((long)xPic * cyClient) / (long)yPic;
    yPic = cyClient;
  }
  else
  {
    //    
    yPic = ((long)cxClient * yPic) / (long)xPic;
    xPic = cxClient;
  }
  ReleaseDC (0, hDC);

  //  
  *width = xPic; *height = yPic;
}

