// ----------------------------------------
//       Clipboard
//      Clipboard
// ----------------------------------------
#define STRICT
#include <windows.h>
#include <mem.h>
#include "cliptxt.hpp"

BOOL InitApp(HINSTANCE);
LRESULT CALLBACK _export WndProc(HWND, UINT, WPARAM, LPARAM);

char const szClassName[]   = "CliptxtClass";
char const szWindowTitle[] = "Clipboard Demo";
char const szClipboardText[] =
  "   \r\n"
  "    Clipboard\r\n"
  " CLIPTXT";

// =====================================
//  WinMain
// =====================================
#pragma argsused
int PASCAL
WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
        LPSTR lpszCmdLine, int nCmdShow)
{
  MSG  msg;   //     
  HWND hwnd;  //    

  //     
  if(!hPrevInstance)
    if(!InitApp(hInstance))
      return FALSE;

  hwnd = CreateWindow(
    szClassName, szWindowTitle,
    WS_OVERLAPPEDWINDOW,
    CW_USEDEFAULT, CW_USEDEFAULT,
    CW_USEDEFAULT, CW_USEDEFAULT,
    0, 0, hInstance, NULL);

  if(!hwnd)
    return FALSE;

  ShowWindow(hwnd, nCmdShow);
  UpdateWindow(hwnd);

  while(GetMessage(&msg, 0, 0, 0))
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }
  return msg.wParam;
}

// =====================================
//  InitApp
//    
// =====================================
BOOL
InitApp(HINSTANCE hInstance)
{
  ATOM aWndClass; //    
  WNDCLASS wc;    //   
                  //  
  memset(&wc, 0, sizeof(wc));
  wc.lpszMenuName  = "APP_MENU";
  wc.style         = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc   = (WNDPROC) WndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon(hInstance, "APP_ICON");
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
  wc.lpszClassName = (LPSTR)szClassName;

  aWndClass = RegisterClass(&wc);
  return (aWndClass != 0);
}

// =====================================
//  WndProc
// =====================================
LRESULT CALLBACK _export
WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  HDC hdc;
  PAINTSTRUCT ps;
  RECT rc;

  static HGLOBAL hglbTextCopyBuf;
  LPSTR lpTextCopy;
  static HGLOBAL hglbTextPasteBuf;
  LPSTR lpTextPaste;
  static HGLOBAL hglbClipBuf;
  LPSTR lpClipBuf;

  switch (msg)
  {
    case WM_CREATE:
    {
      //     ,
      //   Clipboard.
      //     - 1 
      hglbTextPasteBuf = GlobalAlloc(GHND, 1);
      if(hglbTextPasteBuf == NULL)
        return -1;

      //        
      lpTextPaste = (LPSTR)GlobalLock(hglbTextPasteBuf);
      if(hglbTextPasteBuf == NULL)
        return -1;

      //     
      lpTextPaste[0] = '\0';

      //  
      GlobalUnlock(hglbTextPasteBuf);
      return 0;
    }
    case WM_PAINT:
    {
      hdc = BeginPaint(hwnd, &ps);

      //     ,
      //      
      //     
      lpTextPaste = (LPSTR)GlobalLock(hglbTextPasteBuf);
      if(lpTextPaste != NULL)
      {
        GetClientRect(hwnd, &rc);
        DrawText(hdc, lpTextPaste, -1, &rc,
          DT_LEFT | DT_EXPANDTABS);
        GlobalUnlock(hglbTextPasteBuf);
      }
      EndPaint(hwnd, &ps);
    }
    case WM_COMMAND:
    {
      switch (wParam)
      {
        //     Clipboard
        case CM_EDITCOPY:
        {
          //      
          hglbTextCopyBuf = GlobalAlloc(GHND,
            sizeof(szClipboardText) + 1);
          if(hglbTextCopyBuf != NULL)
          {
            //   
            lpTextCopy = (LPSTR)GlobalLock(hglbTextCopyBuf);
            if(lpTextCopy != NULL)
            {
              //      
              lstrcpy(lpTextCopy, szClipboardText);

              //   
              GlobalUnlock(hglbTextCopyBuf);

              //  Clipboard   
              OpenClipboard(hwnd);
              EmptyClipboard();

              //    Clipboard
              SetClipboardData(CF_TEXT, hglbTextCopyBuf);

              //  Clipboard
              CloseClipboard();
            }
            else
              MessageBox(hwnd, " ",
                (LPSTR)szWindowTitle, MB_OK | MB_ICONHAND);
          }
          else
            MessageBox(hwnd, " ",
              (LPSTR)szWindowTitle, MB_OK | MB_ICONHAND);
          return 0;
        }

        //     Clipboard
        case CM_EDITPASTE:
        {
          //  Clipboard
          OpenClipboard(hwnd);

          //    ,
          //    Clipboard
          hglbClipBuf = GetClipboardData(CF_TEXT);

          //   Clipboard    
          // ,  
          if(hglbClipBuf != NULL)
          {
            //    Clipboard
            lpClipBuf = (LPSTR)GlobalLock(hglbClipBuf);
            if(lpClipBuf != NULL)
            {
              //   ,  
              //  ,   Clipboard,
              //     
              //  Clipboard
              hglbTextPasteBuf = GlobalReAlloc(hglbTextPasteBuf,
                 GlobalSize(hglbClipBuf), GMEM_NODISCARD);

              //        
              //  Clipboard
              lpTextPaste = (LPSTR)GlobalLock(hglbTextPasteBuf);
              if(lpTextPaste != NULL)
              {
                lstrcpy(lpTextPaste, lpClipBuf);

                //    
                InvalidateRect(hwnd, NULL, TRUE);

                //  
                GlobalUnlock(hglbTextPasteBuf);
              }
              else
                MessageBox(hwnd, " ",
                  (LPSTR)szWindowTitle, MB_OK | MB_ICONHAND);

              //    Clipboard 
              GlobalUnlock(hglbClipBuf);
            }
            else
              MessageBox(hwnd, " ",
                (LPSTR)szWindowTitle, MB_OK | MB_ICONHAND);
          }
          else
            MessageBox(hwnd, " CF_TEXT ",
            (LPSTR)szWindowTitle, MB_OK | MB_ICONHAND);

          //  Clipboard
          CloseClipboard();
          return 0;
        }

        case CM_HELPABOUT:
        {
	       MessageBox(hwnd,
            " CLIPTXT\n(C)  .., 1995",
	         (LPSTR)szWindowTitle, MB_OK | MB_ICONINFORMATION);
          return 0;
        }
        case CM_FILEEXIT:
        {
          DestroyWindow(hwnd);
          return 0;
        }
        default:
          return 0;
      }
    }
    case WM_DESTROY:
    {
      //  ,   
      // ,   Clipboard
      if(hglbTextPasteBuf != NULL)
        GlobalFree(hglbTextPasteBuf);

      PostQuitMessage(0);
      return 0;
    }
    default:
      break;
  }
  return DefWindowProc(hwnd, msg, wParam, lParam);
}