// ----------------------------------------
//  GETCOLOR
//  ,    
// ----------------------------------------

#define STRICT
#include <windows.h>
#include <windowsx.h>
#include <commdlg.h>
#include <mem.h>

//  
BOOL InitApp(HINSTANCE);
LRESULT CALLBACK _export WndProc(HWND, UINT, WPARAM, LPARAM);
BOOL GetPenColor(HWND, COLORREF *);

//   
char const szClassName[]   = "GetColorClass";

//  
char const szWindowTitle[] = "Get Color";

//   
HINSTANCE hInst;

//    
HBRUSH hbrush, hbrushOldBrush;
HPEN hpen, hpenOldPen;

// =====================================
//  WinMain
// =====================================
#pragma argsused

int PASCAL
WinMain(HINSTANCE hInstance, 
	HINSTANCE hPrevInstance,
	LPSTR     lpszCmdLine, 
	int       nCmdShow)
{
  MSG  msg;   //     
  HWND hwnd;  //    

  //  
  if(!InitApp(hInstance))
      return FALSE;

  //    
  hInst = hInstance;

  //    
  hwnd = CreateWindow(
    szClassName,         //   
    szWindowTitle,       //  
    WS_OVERLAPPEDWINDOW, //  
    CW_USEDEFAULT,       //    
    CW_USEDEFAULT,       // ,   
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    0, 0, hInstance, NULL);      
                       
  //     ,  
  if(!hwnd)
    return FALSE;

  //   
  ShowWindow(hwnd, nCmdShow);
  UpdateWindow(hwnd);

  //    
  while(GetMessage(&msg, 0, 0, 0))
  {
    DispatchMessage(&msg);
  }

  return msg.wParam;
}

// =====================================
//  InitApp
//    
// =====================================

BOOL
InitApp(HINSTANCE hInstance)
{
  ATOM aWndClass; //    
  WNDCLASS wc;    //   
                  //  
  //       
  memset(&wc, 0, sizeof(wc));

  //      
  wc.hbrBackground = (HBRUSH)(COLOR_APPWORKSPACE + 1);

  wc.lpszMenuName  = NULL;
  wc.style         = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc   = (WNDPROC) WndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.lpszClassName = (LPSTR)szClassName;

  //  
  aWndClass = RegisterClass(&wc);

  return (aWndClass != 0);
}

// =====================================
//  WndProc
// =====================================

LRESULT CALLBACK _export
WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  HDC hdc;
  PAINTSTRUCT ps;

  switch (msg)
  {
    case WM_CREATE:
    {
      //       
      hbrush =
        CreateSolidBrush(GetSysColor(COLOR_ACTIVECAPTION));
      hpen  = CreatePen(PS_SOLID, 10, RGB(0,0,0));
      return 0;
    }

    case WM_SYSCOLORCHANGE:
    {
      //    ,  
      //    
      if(hbrush)
      {
        DeleteBrush(hbrush);
        hbrush =
          CreateSolidBrush(GetSysColor(COLOR_ACTIVECAPTION));
      }
      return 0;
    }

    //   
    case WM_PAINT:
    {
      RECT rc;

      //    
      //      
      hdc = BeginPaint(hwnd, &ps);

      //      
      hbrushOldBrush = SelectBrush(hdc, hbrush);
      hpenOldPen = SelectPen(hdc, hpen);

      //  
      Rectangle(hdc, 10, 20, 300, 50);

      //      
      SelectBrush(hdc, hbrushOldBrush);
      SelectPen(hdc, hpenOldPen);

      //   
      EndPaint(hwnd, &ps);
      return 0;
    }

    case WM_LBUTTONDOWN:
    {
      COLORREF clrref;

      //     
      if(GetPenColor(hwnd, &clrref))
      {
        //    ,  ,
        //    
        DeletePen(hpen);
        hpen  = CreatePen(PS_SOLID, 10, clrref);

        //  
        InvalidateRect(hwnd, NULL, TRUE);
      }
      return 0;
    }

    case WM_DESTROY:
    {
      //      
      DeleteBrush(hbrush);
      DeletePen(hpen);

      PostQuitMessage(0);
      return 0;
    }

    default:
      break;
  }
  return DefWindowProc(hwnd, msg, wParam, lParam);
}

// ---------------------------------------------------
//  GetPenColor
//   
// ---------------------------------------------------

BOOL GetPenColor(HWND hwnd, COLORREF *clrref)
{
  CHOOSECOLOR cc;
  COLORREF aclrCust[16];
  int i;

  //   
  // "Custom Colors"
  for (i = 0; i < 16; i++)
    aclrCust[i] = RGB(255, 255, 255);

  //     
  //    CHOOSECOLOR
  memset(&cc, 0, sizeof(CHOOSECOLOR));

  //   
  cc.lStructSize = sizeof(CHOOSECOLOR);
  cc.hwndOwner = hwnd;
  cc.rgbResult = RGB(0, 0, 0);
  cc.lpCustColors = aclrCust;
  cc.Flags = 0;

  //     
  if (ChooseColor(&cc))
  {
    *clrref = cc.rgbResult;
    return TRUE;
  }
  else
    return FALSE;
}
