// ----------------------------------------------
//      
// ----------------------------------------------

#define STRICT
#include <windows.h>
#include <mem.h>
#include "gmenu.hpp"

//  
BOOL InitApp(HINSTANCE);
LRESULT CALLBACK _export WndProc(HWND, UINT, WPARAM, LPARAM);

//   
char const szClassName[]   = "GMenuClass";

//  
char const szWindowTitle[] = "Menu Demo";

//    
HMENU hmenu;

//   
HMENU hmenuFile;      // "File"
HMENU hmenuLineStyle; // "Edit"
HMENU hmenuHelp;      // "Help"

//      
HBITMAP hbmpLine1;
HBITMAP hbmpLine2;
HBITMAP hbmpLine3;
HBITMAP hbmpLine4;
HBITMAP hbmpLineStyle;
HBITMAP hbmpChecked;
HBITMAP hbmpUnchecked;

//    
HINSTANCE hInst;

// =====================================
//  WinMain
// =====================================
#pragma argsused

int PASCAL
WinMain(HINSTANCE hInstance, 
	HINSTANCE hPrevInstance,
	LPSTR     lpszCmdLine, 
	int       nCmdShow)
{
  MSG  msg;   //     
  HWND hwnd;  //    

  //  
  if(!InitApp(hInstance))
      return FALSE;

  //     
  hInst = hInstance;

  //     
  //   
  hwnd = CreateWindow(
    szClassName,         //   
    szWindowTitle,       //  
    WS_OVERLAPPEDWINDOW, //  
    CW_USEDEFAULT,       //    
    CW_USEDEFAULT,       // ,   
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    0,                   //   
    0,                   //  
    hInstance,           //  
    NULL);               //   
                         // 
  //     ,  
  if(!hwnd)
    return FALSE;

  //   
  ShowWindow(hwnd, nCmdShow);
  UpdateWindow(hwnd);

  //    
  while(GetMessage(&msg, 0, 0, 0))
  {
    DispatchMessage(&msg);
  }
  return msg.wParam;
}

// =====================================
//  InitApp
//    
// =====================================

BOOL
InitApp(HINSTANCE hInstance)
{
  ATOM aWndClass; //    
  WNDCLASS wc;    //   
                  //  

  //       
  memset(&wc, 0, sizeof(wc));

  wc.lpszMenuName  = NULL;
  wc.style         = 0;
  wc.lpfnWndProc   = (WNDPROC) WndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
  wc.lpszClassName = (LPSTR)szClassName;

  //  
  aWndClass = RegisterClass(&wc);
  return (aWndClass != 0);
}

// =====================================
//  WndProc
// =====================================

LRESULT CALLBACK _export
WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  switch (msg)
  {
    case WM_CREATE:
    {
      //    
      hbmpLine1     = LoadBitmap(hInst, "LINE1");
      hbmpLine2     = LoadBitmap(hInst, "LINE2");
      hbmpLine3     = LoadBitmap(hInst, "LINE3");
      hbmpLine4     = LoadBitmap(hInst, "LINE4");
      hbmpLineStyle = LoadBitmap(hInst, "LINESTYLE");
      hbmpChecked   = LoadBitmap(hInst, "CHECKED");
      hbmpUnchecked = LoadBitmap(hInst, "UNCHECKED");

      //     
      hmenu = CreateMenu();

      //      
      SetMenu(hwnd, hmenu);

      //   
      hmenuFile = CreatePopupMenu();
      hmenuHelp = CreatePopupMenu();
      hmenuLineStyle = CreatePopupMenu();

      //     "File"
      AppendMenu(hmenuFile, MF_ENABLED | MF_STRING,
         CM_FILENEW,    "&New");
      AppendMenu(hmenuFile, MF_ENABLED | MF_STRING,
         CM_FILEOPEN,   "&Open");
      AppendMenu(hmenuFile, MF_ENABLED | MF_STRING,
         CM_FILECLOSE,  "&Close");
      AppendMenu(hmenuFile, MF_ENABLED | MF_STRING,
         CM_FILESAVE,   "&Save");
      AppendMenu(hmenuFile, MF_ENABLED | MF_STRING,
         CM_FILESAVEAS, "Save &as...");
      AppendMenu(hmenuFile, MF_SEPARATOR, 0, NULL);
      AppendMenu(hmenuFile, MF_DISABLED | MF_STRING,
         CM_FILEDEMO, "&Demo Version");
      AppendMenu(hmenuFile, MF_SEPARATOR, 0, NULL);
      AppendMenu(hmenuFile, MF_ENABLED | MF_STRING,
         CM_FILEEXIT, "E&xit");

      //   "Demo Version"  "File" 
      // ,    
      //       
      SetMenuItemBitmaps(hmenuFile, CM_FILEDEMO, MF_BYCOMMAND,
        hbmpUnchecked, hbmpChecked);

      //   "Demo Version"
      CheckMenuItem(hmenuFile, CM_FILEDEMO,
         MF_BYCOMMAND | MF_CHECKED);

      //     "Line Style".
      //     
      //  bitmap 
      AppendMenu(hmenuLineStyle, MF_ENABLED | MF_BITMAP,
         CM_LINE1, (LPCSTR)(DWORD)hbmpLine1);
      AppendMenu(hmenuLineStyle, MF_ENABLED | MF_BITMAP,
         CM_LINE2, (LPCSTR)(DWORD)hbmpLine2);
      AppendMenu(hmenuLineStyle, MF_ENABLED | MF_BITMAP,
         CM_LINE3, (LPCSTR)(DWORD)hbmpLine3);
      AppendMenu(hmenuLineStyle, MF_ENABLED | MF_BITMAP,
         CM_LINE4, (LPCSTR)(DWORD)hbmpLine4);

      //     "Help"
      AppendMenu(hmenuHelp, MF_GRAYED | MF_STRING,
         CM_HELPINDEX, "&Index\tF1");
      AppendMenu(hmenuHelp, MF_GRAYED | MF_STRING,
         CM_HELPKEYBOARD, "&Keyboard");
      AppendMenu(hmenuHelp, MF_GRAYED | MF_STRING,
         CM_HELPCOMMANDS, "&Commands");
      AppendMenu(hmenuHelp, MF_GRAYED | MF_STRING,
         CM_HELPPROCEDURES, "&Procedures");
      AppendMenu(hmenuHelp, MF_GRAYED | MF_STRING,
         CM_HELPUSING_HELP, "&Using help");
      AppendMenu(hmenuHelp, MF_SEPARATOR, 0, NULL);
      AppendMenu(hmenuHelp, MF_ENABLED | MF_STRING,
         CM_HELPABOUT, "&About...");

      //       
      AppendMenu(hmenu, MF_ENABLED | MF_POPUP,
         (UINT)hmenuFile, "&File");

      //    "Line Style" 
      //  bitmap
      AppendMenu(hmenu, MF_ENABLED | MF_POPUP | MF_BITMAP,
         (UINT)hmenuLineStyle, (LPCSTR)(DWORD)hbmpLineStyle);

      AppendMenu(hmenu, MF_ENABLED | MF_POPUP,
         (UINT)hmenuHelp, "&Help");

      //  
      DrawMenuBar(hwnd);
      return 0;
    }

    case WM_COMMAND:
    {
      switch (wParam)
      {
        //   
	case CM_HELPUSING_HELP:
        case CM_HELPPROCEDURES:
        case CM_HELPCOMMANDS:
        case CM_HELPKEYBOARD:
        case CM_HELPINDEX:
        case CM_FILESAVEAS:
        case CM_FILESAVE:
	case CM_FILEOPEN:
	case CM_FILENEW:
	case CM_FILECLOSE:
	{
          //    
	  MessageBox(hwnd, "  ",
	          NULL, MB_OK);
          return 0;
	}

        //   "About..."   "Help"
	case CM_HELPABOUT:
	{
	  MessageBox(hwnd, " GMENU\n(C)  .., 1994",
		  szWindowTitle, MB_OK | MB_ICONINFORMATION);
          return 0;
	}

	//   
        case CM_FILEEXIT:
        {
	  DestroyWindow(hwnd);
          return 0;
        }

        default:
          return 0;
      }
    }

    case WM_DESTROY:
    {
      //    
      DestroyMenu(hmenuFile);
      DestroyMenu(hmenuHelp);
      DestroyMenu(hmenuLineStyle);
      DestroyMenu(hmenu);

      //  
      DeleteObject(hbmpLine1);
      DeleteObject(hbmpLine2);
      DeleteObject(hbmpLine3);
      DeleteObject(hbmpLine4);
      DeleteObject(hbmpLineStyle);
      DeleteObject(hbmpChecked);
      DeleteObject(hbmpUnchecked);

      PostQuitMessage(0);
      return 0;
    }

    default:
      break;
  }
  return DefWindowProc(hwnd, msg, wParam, lParam);
}