// ------------------------------------------------
//  MCIWNDC
//    MCIWnd  
//     Multimedia
// ------------------------------------------------

#define STRICT
#include <windows.h>
#include <windowsx.h>
#include <commdlg.h>
#include <memory.h>
#include <vfw.h>
#include "mciwndc.h"

//  
BOOL    InitApp(HINSTANCE);
LRESULT CALLBACK _export WndProc(HWND, UINT, WPARAM, LPARAM);
BOOL mciwndSelectFile(LPSTR lpszFileName);

//  
char const szClassName[]   = "MCIWNDCClass";
char const szMovieClass[]  = MCIWND_WINDOW_CLASS;
char const szWindowTitle[] = "MCIWnd Player & Recorder";
HINSTANCE  hInst;
HWND hwndMovie = NULL;

// =====================================
//  WinMain
// =====================================
int PASCAL
WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
        LPSTR lpszCmdLine, int nCmdShow)
{
  MSG  msg;      //     
  HWND hwnd;     //    
  WORD wVersion; //  Video for Windows

  if(hPrevInstance)
    return FALSE;

  //   Video for Windows
  wVersion = HIWORD(VideoForWindowsVersion());
  if(wVersion < 0x010a)
  {
    MessageBox(NULL, " Video for Windows"
        "  1.1    ",
        "MCIWnd Error", MB_OK | MB_ICONHAND);
    return FALSE;
  }

  if(!InitApp(hInstance))
    return FALSE;
    
  hInst = hInstance;

  hwnd = CreateWindow(
    szClassName,         //   
    szWindowTitle,       //  
    WS_OVERLAPPEDWINDOW, //  
    CW_USEDEFAULT,       //    
    CW_USEDEFAULT,       
    400, 350, 0, 0, hInstance, NULL);
                       
  if(!hwnd)
    return FALSE;

  ShowWindow(hwnd, nCmdShow);
  UpdateWindow(hwnd);

  while(GetMessage(&msg, 0, 0, 0))
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }
  return msg.wParam;
}

// =====================================
//  InitApp
//    
// =====================================

BOOL InitApp(HINSTANCE hInstance)
{
  ATOM aWndClass; //    
  WNDCLASS wc;    //   

  memset(&wc, 0, sizeof(wc));
  wc.lpszMenuName  = "APP_MENU";
  wc.style         = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc   = (WNDPROC) WndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon(hInstance, "APPICON");
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
  wc.lpszClassName = (LPSTR)szClassName;

  aWndClass = RegisterClass(&wc);
  return (aWndClass != 0);
}

// =====================================
//  WndProc
// =====================================

LRESULT CALLBACK _export
WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  switch (msg)
  {
// ------------------------------------------------------------
// WM_INITMENU
//  
// ------------------------------------------------------------
    case WM_INITMENU:
    {                                       
      //    MCI  
      WORD wStyles = MCIWndGetStyles(hwndMovie);
      
      //   MCI   ,
      //   "Play Bar"  "Styles"
      CheckMenuItem(GetMenu(hwnd), CM_STPBAR, 
        (wStyles & MCIWNDF_NOPLAYBAR) ? MF_UNCHECKED : MF_CHECKED);
      return 0;
    }

// ------------------------------------------------------------
// WM_COMMAND
//    
// ------------------------------------------------------------
    case WM_COMMAND:
    {
      switch (wParam)
      {
        // -------------------------------------------------
        //  "About"  "Help"
        // -------------------------------------------------
        case CM_HELPABOUT:
        {
          MessageBox(hwnd,
            "MCIWnd Player & Recorder, v.1.0\n"
            "(C) Frolov A.V., 1994",
            "About MCIWNDC", MB_OK | MB_ICONINFORMATION);
          return 0;
        }

        // -------------------------------------------------
        //  "New Waveaudio"  "File"
        // -------------------------------------------------
        case CM_FILENEW:
        {
          //   MCI   ,  
          if(hwndMovie)
            MCIWndDestroy(hwndMovie);

          //      ,
          //    waveaudio
          hwndMovie = MCIWndCreate(
            hwnd, hInst,
            WS_VISIBLE | WS_CHILD | WS_BORDER |
            MCIWNDF_RECORD, 
            (LPSTR)"waveaudio");
             
          //   
          MCIWndNew(hwndMovie, "waveaudio");
          return 0;
        }

        // -------------------------------------------------
        //  "Save Waveaudio As..."  "File"
        // -------------------------------------------------
        case CM_FILESAVEAS:
        {
          //    "Save As..." 
          //  
          MCIWndSaveDialog(hwndMovie);
          return 0;
        }

        // -------------------------------------------------
        //  "CD Audio"  "File"
        // -------------------------------------------------
        case CM_FILECDAUDIO:
        {
          //   MCI   ,  
          if(hwndMovie)
            MCIWndDestroy(hwndMovie);

          //      CD,
          //    cdaudio
          hwndMovie = MCIWndCreate(
            hwnd, hInst,
            WS_VISIBLE | WS_CHILD | WS_BORDER |
            MCIWNDF_RECORD, 
            (LPSTR)"cdaudio");
          return 0;
        }

        // -------------------------------------------------
        //  "Open"  "File"
        // -------------------------------------------------
        case CM_FILEOPEN:
        {
          char szBuff[256];
          
          //   MCI   ,  
          if(hwndMovie)
            MCIWndDestroy(hwndMovie);
          
          //       MCI
          if(mciwndSelectFile(szBuff))
          {
             //   MCI
             hwndMovie = MCIWndCreate(
               hwnd, hInst,
               WS_VISIBLE | WS_CHILD | WS_BORDER |
               MCIWNDF_RECORD,
               (LPSTR)szBuff);
          }
          return 0;
        }

        // -------------------------------------------------
        //  "Close"  "File"
        // -------------------------------------------------
        case CM_FILECLOSE:
        {
          //   MCI
          MCIWndDestroy(hwndMovie);
          hwndMovie = NULL;
          return 0;
        }

        // -------------------------------------------------
        //  "Movie"
        // -------------------------------------------------
        case CM_MVIPLAY:    // "Play"
        {
          // 
          MCIWndPlay(hwndMovie);
          return 0;
        }

        case CM_MVIRPLAY:   // "Play Reverse"
        {
          //    
          MCIWndPlayReverse(hwndMovie);
          return 0;
        }

        case CM_MVISTOP:    // "Stop"
        {
          // 
          MCIWndStop(hwndMovie);
          return 0;
        }

        case CM_MVIRECORD:  // "Record"
        {
          // 
          MCIWndRecord(hwndMovie);
          return 0;
        }             
        
        case CM_MVIHOME:    // "Home"
        {
          //   
          MCIWndHome(hwndMovie);
          return 0;
        }

        case CM_MVIEND:     // "End"
        {
          //   
          MCIWndEnd(hwndMovie);
          return 0;
        }

        case CM_MVISTEP:    // "Step Fwrd"
        {
          //  
          MCIWndStep(hwndMovie, 1);
          return 0;
        }

        case CM_MVIRSTEP:   // "Step Back"
        {
          //  
          MCIWndStep(hwndMovie, -1);
          return 0;
        }

        case CM_MVIINFO:    // "Info..."
        {              
          char szBuff[512], szBuff1[256];
          long dwSize;
          
          //   MCI ,  
          //     
          if(hwndMovie)
          {
            //  
            MCIWndGetDevice(hwndMovie, (LPSTR)szBuff, 512);
            lstrcat(szBuff, (LPSTR)"\n");
          
            //      
            MCIWndGetFileName(hwndMovie, (LPSTR)szBuff1, 256);
            lstrcat(szBuff, (LPSTR)szBuff1);
            lstrcat(szBuff, (LPSTR)"\n");

            //  
            dwSize = MCIWndGetLength(hwndMovie);
            wsprintf(szBuff1, "Size: %ld ", (long)dwSize);
            lstrcat(szBuff, (LPSTR)szBuff1);

            //  
            MCIWndGetTimeFormat(hwndMovie, (LPSTR)szBuff1, 256);
            lstrcat(szBuff, (LPSTR)szBuff1);

            MessageBox(hwnd, szBuff,
              "Media Info", MB_OK | MB_ICONINFORMATION);
          }
          return 0;
        }
        // -------------------------------------------------
        //  "Styles"
        // -------------------------------------------------
        case CM_STPBAR:     // "Play Bar"
        {
          //    MCI
          WORD wStyles = MCIWndGetStyles(hwndMovie);
      
          //    MCIWNDF_NOPLAYBAR
          MCIWndChangeStyles(hwndMovie, MCIWNDF_NOPLAYBAR, 
            (wStyles & MCIWNDF_NOPLAYBAR) ? 0 : MCIWNDF_NOPLAYBAR);
          return 0;
        }

        // -------------------------------------------------
        //  "Exit"  "File"
        //   
        // -------------------------------------------------
        case CM_FILEEXIT:
        {
          DestroyWindow(hwnd);
          return 0;
        }
        default:
          return 0;
      }
    }

    //     
    case WM_PALETTECHANGED:
    {
      SendMessage(hwndMovie, msg, wParam, lParam);
      break;
    }
    case WM_QUERYNEWPALETTE:
    {
      return SendMessage(hwndMovie, msg, wParam, lParam);
    }


// ------------------------------------------------------------
// WM_DESTROY
//    
// ------------------------------------------------------------
    case WM_DESTROY:
    {
      PostQuitMessage(0);
      return 0;
    }
    default:
      break;
  }
  return DefWindowProc(hwnd, msg, wParam, lParam);
}

//-----------------------------------------------------
// mciwndSelectFile
//  
//-----------------------------------------------------
BOOL mciwndSelectFile(LPSTR lpszFileName)
{
  OPENFILENAME ofn;

  char szFile[256];
  char szFileTitle[256];
  char szFilter[256] =
     "Video Files\0*.avi\0"
     "Waveaudio Files\0*.wav\0"
     "MIDI Files\0*.mid;*.rmi\0"
     "Any Files\0*.*\0";
  szFile[0] = '\0';
  memset(&ofn, 0, sizeof(OPENFILENAME));

  //    
  ofn.lStructSize       = sizeof(OPENFILENAME);
  ofn.hwndOwner         = NULL;
  ofn.lpstrFilter       = szFilter;
  ofn.nFilterIndex      = 1;
  ofn.lpstrFile         = szFile;
  ofn.nMaxFile          = sizeof(szFile);
  ofn.lpstrFileTitle    = szFileTitle;
  ofn.nMaxFileTitle     = sizeof(szFileTitle);
  ofn.lpstrInitialDir   = NULL;
  ofn.Flags =   OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST
               | OFN_HIDEREADONLY;
  //   
  if (GetOpenFileNamePreview(&ofn))
  {
    //     
    lstrcpy(lpszFileName, (LPSTR)szFile);
    return TRUE;
  }
  else
    return FALSE;
}
