// ------------------------------------------------
//  MIDIPL
//   MIDI
//     MCI
// ------------------------------------------------

#define STRICT
#include <windows.h>
#include <windowsx.h>
#include <mmsystem.h>
#include <mem.h>
#pragma hdrstop

#include "midipl.hpp"
#include "midiio.hpp"

//  
#define BEEP_TIMER 1

//   
#define ID_SCROLL 10

//   
#define SCROLL_SIZE 400

//  
BOOL    InitApp(HINSTANCE);
LRESULT CALLBACK _export WndProc(HWND, UINT, WPARAM, LPARAM);

//  
int      nMode = MODE_STOP;
MMTIME   mmtimeOut;
BOOL     fFileLoaded = FALSE;
int      nPosition;
HWND     hScroll;
UINT     wOutDeviceID;
BYTE     szFileName[128];
DWORD    dwFileSize;

char const szClassName[]   = "MCIMIDIClass";
char const szWindowTitle[] = "MIDI Player";
HINSTANCE  hInst;

// =====================================
//  WinMain
// =====================================
#pragma argsused

int PASCAL
WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
        LPSTR lpszCmdLine, int nCmdShow)
{
  MSG  msg;   //     
  HWND hwnd;  //    

  if(hPrevInstance)
    return FALSE;

  if(!InitApp(hInstance))
    return FALSE;

  hInst = hInstance;

  hwnd = CreateWindow(
    szClassName,         //   
    szWindowTitle,       //  
    WS_OVERLAPPEDWINDOW, //  
    CW_USEDEFAULT,       //    
    CW_USEDEFAULT,       
    450, 120, 0, 0, hInstance, NULL);
                       
  if(!hwnd)
    return FALSE;

  ShowWindow(hwnd, nCmdShow);
  UpdateWindow(hwnd);

  while(GetMessage(&msg, 0, 0, 0))
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }
  return msg.wParam;
}

// =====================================
//  InitApp
//    
// =====================================

BOOL
InitApp(HINSTANCE hInstance)
{
  ATOM aWndClass; //    
  WNDCLASS wc;    //   

  memset(&wc, 0, sizeof(wc));
  wc.lpszMenuName  = "APP_MENU";
  wc.style         = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc   = (WNDPROC) WndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon(hInstance, "APPICON");
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
  wc.lpszClassName = (LPSTR)szClassName;

  aWndClass = RegisterClass(&wc);
  return (aWndClass != 0);
}

// =====================================
//  WndProc
// =====================================

LRESULT CALLBACK _export
WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  HDC hdc;
  PAINTSTRUCT ps;
  int rc;

  switch (msg)
  {
// ------------------------------------------------------------
// WM_CREATE
//    
// ------------------------------------------------------------
    case WM_CREATE:
    {
       nMode        = MODE_STOP;
       fFileLoaded  = FALSE;
       wOutDeviceID = 0;

       //  
       SetTimer(hwnd, BEEP_TIMER, 100, NULL);

       //   
       hScroll = CreateWindow("scrollbar", NULL,
         WS_CHILD | WS_VISIBLE | SBS_HORZ,
         10, 40, SCROLL_SIZE, 15, hwnd,
         (HMENU) ID_SCROLL, hInst, NULL);

       //   
       nPosition = 0;

       //    
       //    
       SetScrollRange(hScroll, SB_CTL, 1, SCROLL_SIZE, TRUE);

       //  
       SetScrollPos(hScroll, SB_CTL, nPosition, TRUE);
       return 0;
    }

// ------------------------------------------------------------
// WM_PAINT
//   
// ------------------------------------------------------------
    case WM_PAINT:
    {
      //    
      //      
      hdc = BeginPaint(hwnd, &ps);

      //    
      if(nMode == MODE_STOP)
        TextOut(hdc, 10, 10, "", 11);
      else if(nMode == MODE_PLAYING)
        TextOut(hdc, 10, 10, " ...", 20);
      else if(nMode == MODE_PLAYINGPAUSED)
        TextOut(hdc, 10, 10, " ", 24);
      else
        TextOut(hdc, 10, 10, " !", 19);

      //   
      EndPaint(hwnd, &ps);
      return 0;
    }

// ------------------------------------------------------------
// WM_COMMAND
//    
// ------------------------------------------------------------
    case WM_COMMAND:
    {
      switch (wParam)
      {
        // -------------------------------------------------
        //  "About"  "Help"
        // -------------------------------------------------
        case CM_HELPABOUT:
        {
          MessageBox(hwnd,
            "MIDI Player, v.1.0\n"
            "(C) Frolov A.V., 1994",
            "About MIDIPL", MB_OK | MB_ICONINFORMATION);
          return 0;
        }

        // -------------------------------------------------
        //  "Open"  "File"
        // -------------------------------------------------
        case CM_FILEOPEN:
        {
          char szTitle[256];

          //   
          if(!mcimidiSelectFile(szFileName))
            return 0;

          //       
          lstrcpy(szTitle, szWindowTitle);
          lstrcat(szTitle, " - ");
          lstrcat(szTitle, szFileName);
          SetWindowText(hwnd, szTitle);

          //    ,
          //      
          if(wOutDeviceID)
          {
            mcimidiStop(wOutDeviceID);
            mcimidiClose(wOutDeviceID);
            wOutDeviceID = 0;

            //  
            nMode = MODE_STOP;

            //     ,
            //   
            InvalidateRect(hwnd, NULL, TRUE);
          }

          //      
          nPosition = 0;
          SetScrollPos(hScroll, SB_CTL, nPosition, TRUE);

          //    
          fFileLoaded = TRUE;
          return 0;
        }

        // -------------------------------------------------
        //  "Play!"
        //    MIDI
        // -------------------------------------------------
        case CM_CTLPLAY:
        {
          //      ,
          //   
          if((fFileLoaded == TRUE) && (nMode == MODE_STOP))
          {
            //  
            nMode = MODE_PLAYING;

            //     ,
            //   
            InvalidateRect(hwnd, NULL, TRUE);

            //      ,
            //  
            if(!wOutDeviceID)
              wOutDeviceID = mcimidiOpen((LPSTR)szFileName);

            //  
            mcimidiPlay(hwnd, wOutDeviceID);
          }
          return 0;
        }

        // -------------------------------------------------
        //  "Stop!"
        //      MIDI
        // -------------------------------------------------
        case CM_CTLSTOP:
        {
          if(nMode == MODE_PLAYING || nMode == MODE_PLAYINGPAUSED)
          {
            //  
            mcimidiStop(wOutDeviceID);
          }

          //      
          nPosition = 0;
          SetScrollPos(hScroll, SB_CTL, nPosition, TRUE);

          //  
          nMode = MODE_STOP;
          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }

        // -------------------------------------------------
        //  "Pause!"
        //   
        // -------------------------------------------------
        case CM_CTLPAUSE:
        {
          if(nMode == MODE_PLAYING)
          {
            //   
            mcimidiPause(wOutDeviceID);
            nMode = MODE_PLAYINGPAUSED;
          }

          InvalidateRect(hwnd, NULL, TRUE);
          return 0;
        }

        // -------------------------------------------------
        //  "Resume!"
        //    
        // -------------------------------------------------
        case CM_CTLRESUME:
        {
          if(nMode == MODE_PLAYINGPAUSED)
          {
            //  
            mcimidiPlayCurrent(hwnd, wOutDeviceID);
            nMode = MODE_PLAYING;
            InvalidateRect(hwnd, NULL, TRUE);
          }
          return 0;
        }

        // -------------------------------------------------
        //  "Exit"  "File"
        //   
        // -------------------------------------------------
        case CM_FILEEXIT:
        {
          DestroyWindow(hwnd);
          return 0;
        }
        default:
          return 0;
      }
    }

// ------------------------------------------------------------
// MM_MCINOTIFY
// ------------------------------------------------------------
    case MM_MCINOTIFY:
    {
      //     , 
      //    
      if(nMode == MODE_PLAYING)
      {
        if(wOutDeviceID)
        {
          mcimidiStop(wOutDeviceID);
          mcimidiClose(wOutDeviceID);
          wOutDeviceID=0;
          nMode = MODE_STOP;
          InvalidateRect(hwnd, NULL, TRUE);
        }
      }
      return 0;
    }

// ------------------------------------------------------------
// WM_TIMER
//   
// ------------------------------------------------------------
    case WM_TIMER:
    {
      MCI_STATUS_PARMS mciStatus;
      DWORD dwPos;

      //  
      if(nMode == MODE_PLAYING)
      {
        //     
        mciStatus.dwItem = MCI_STATUS_POSITION;
        mciSendCommand(wOutDeviceID, MCI_STATUS,
          MCI_STATUS_ITEM, (DWORD)(LPVOID)&mciStatus);
        dwPos = mciStatus.dwReturn;

        //      
        nPosition = ((DWORD)SCROLL_SIZE * dwPos) / dwFileSize;

        //    
        //    1  SCROLL_SIZE
        if(nPosition > SCROLL_SIZE) nPosition = SCROLL_SIZE;
        if(nPosition < 1) nPosition = 1;

        //    
        //       
        SetScrollPos(hScroll, SB_CTL, nPosition, TRUE);
      }
      return 0;
    }

// ------------------------------------------------------------
// WM_DESTROY
//    
// ------------------------------------------------------------
    case WM_DESTROY:
    {
      //     
      KillTimer(hwnd, BEEP_TIMER);
      DestroyWindow(hScroll);

      //     , 
      //     
      if(fFileLoaded)
      {
        if(nMode == MODE_PLAYING || nMode == MODE_PLAYINGPAUSED)
        {
          mcimidiStop(wOutDeviceID);
          mcimidiClose(wOutDeviceID);
        }
        nMode = MODE_STOP;
      }
      PostQuitMessage(0);
      return 0;
    }
    default:
      break;
  }
  return DefWindowProc(hwnd, msg, wParam, lParam);
}

