#define STRICT
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <richedit.h>
#include "resource.h"
#include "afxres.h"
#include "rtfpad.h"

HINSTANCE hInst;
char szAppName[]  = "RtfEditApp";
char szAppTitle[] = "Rich Text Editor RtfPad";
HWND hwndEdit;
HINSTANCE hRTFLib;

// -----------------------------------------------------
//  WinMain
// -----------------------------------------------------
int APIENTRY 
WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
        LPSTR lpCmdLine, int nCmdShow)
{
  WNDCLASSEX wc;
  HWND hWnd;
  MSG msg;
  
  hInst = hInstance;

  // ,       
  hWnd = FindWindow(szAppName, NULL);
  if(hWnd)
  {
    if(IsIconic(hWnd))
	  ShowWindow(hWnd, SW_RESTORE);
	SetForegroundWindow(hWnd);
    return FALSE;
  }

  //   RICHED32.DLL
  hRTFLib = LoadLibrary("RICHED32.DLL");
  if(!hRTFLib)
    return FALSE;

  //   
  memset(&wc, 0, sizeof(wc));
  wc.cbSize = sizeof(WNDCLASSEX);
  wc.hIconSm = LoadImage(hInst,
    MAKEINTRESOURCE(IDI_APPICONSM), IMAGE_ICON, 16, 16, 0);
  wc.style = 0;
  wc.lpfnWndProc = (WNDPROC)WndProc;
  wc.cbClsExtra  = 0;
  wc.cbWndExtra  = 0;
  wc.hInstance = hInst;
  wc.hIcon = LoadImage(hInst,
    MAKEINTRESOURCE(IDI_APPICON), IMAGE_ICON, 32, 32, 0);
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
  wc.lpszMenuName = MAKEINTRESOURCE(IDR_APPMENU);
  wc.lpszClassName = szAppName;
  if(!RegisterClassEx(&wc))
    if(!RegisterClass((LPWNDCLASS)&wc.style))
	  return FALSE;
    
  //    
  hWnd = CreateWindow(szAppName, szAppTitle, WS_OVERLAPPEDWINDOW,
   	 CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInst, NULL);
  if(!hWnd) return(FALSE);

  //       
  ShowWindow(hWnd, nCmdShow);
  UpdateWindow(hWnd);
  while(GetMessage(&msg, NULL, 0, 0))
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }
  return msg.wParam;
}

// -----------------------------------------------------
//  WndProc
// -----------------------------------------------------
LRESULT WINAPI
WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  switch(msg)
  {
    HANDLE_MSG(hWnd, WM_CREATE,     WndProc_OnCreate);
    HANDLE_MSG(hWnd, WM_DESTROY,    WndProc_OnDestroy);
    HANDLE_MSG(hWnd, WM_COMMAND,    WndProc_OnCommand);
    HANDLE_MSG(hWnd, WM_SIZE,       WndProc_OnSize);
    HANDLE_MSG(hWnd, WM_SETFOCUS,   WndProc_OnSetFocus);

	default:
      return(DefWindowProc(hWnd, msg, wParam, lParam));
  }
}

// -----------------------------------------------------
//  WndProc_OnCreate
// -----------------------------------------------------
BOOL WndProc_OnCreate(HWND hWnd, LPCREATESTRUCT lpCreateStruct)
{
  RECT rc;

  //      
  GetClientRect(hWnd, &rc);

  //    Rich Edit
  hwndEdit = CreateWindowEx(0L, "RICHEDIT", "",
    WS_VISIBLE | WS_CHILD | WS_BORDER | WS_HSCROLL | WS_VSCROLL |
    ES_NOHIDESEL | ES_AUTOVSCROLL | ES_MULTILINE | 
    ES_SAVESEL | ES_SUNKEN,
    0, 0, rc.right - rc.left, rc.bottom - rc.top,
    hWnd, (HMENU) IDC_RTFEDIT, hInst, NULL);

  if(hwndEdit == NULL)
    return FALSE;

  //      Rich Edit
  SetFocus(hwndEdit);

  return TRUE;
}

// -----------------------------------------------------
//  WndProc_OnDestroy
// -----------------------------------------------------
#pragma warning(disable: 4098)
void WndProc_OnDestroy(HWND hWnd)
{
  //    Rich Edit
  if(hwndEdit)
    DestroyWindow(hwndEdit);

  //   RICHED32.DLL
  if(hRTFLib)
    FreeLibrary(hRTFLib);

  PostQuitMessage(0);
  return 0L;
}

// -----------------------------------------------------
//  WndProc_OnCommand
// -----------------------------------------------------
#pragma warning(disable: 4098)
void WndProc_OnCommand(HWND hWnd, int id, 
  HWND hwndCtl, UINT codeNotify)
{
  CHARFORMAT cf;
  CHOOSEFONT chfnt;
  LOGFONT lf;
  HDC hDC;
  PARAFORMAT pf; 

  switch (id)
  {
    //   
	case ID_FORMAT_BOLD:
    {
	  cf.cbSize = sizeof(cf);
      
      //   
      SendMessage(hwndEdit, EM_GETCHARFORMAT, TRUE, (LPARAM)&cf);

      //    dwEffects,   
      //     bold ( )
      cf.dwMask = CFM_BOLD;

      //  ,   
      cf.dwEffects ^= CFE_BOLD;
               
      //   
      SendMessage(hwndEdit, EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);

	  return 0L;
	  break;
    }

	//    
    //  
	case ID_FORMAT_ITALIC:
    {
	  cf.cbSize = sizeof(cf);
      SendMessage(hwndEdit, EM_GETCHARFORMAT, TRUE, (LPARAM)&cf);

      cf.dwMask = CFM_ITALIC;
      cf.dwEffects ^= CFE_ITALIC;
      SendMessage(hwndEdit, EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);

	  return 0L;
	  break;
    }

	//    
    //  
	case ID_FORMAT_UNDERLINE:
    {
	  cf.cbSize = sizeof(cf);
      SendMessage(hwndEdit, EM_GETCHARFORMAT, TRUE, (LPARAM)&cf);

      cf.dwMask = CFM_UNDERLINE;
      cf.dwEffects ^= CFE_UNDERLINE;
      SendMessage(hwndEdit, EM_SETCHARFORMAT, 
        SCF_SELECTION, (LPARAM)&cf);

	  return 0L;
	  break;
    }

	//   
	case ID_FORMAT_FONT:
    {
	  cf.cbSize = sizeof(cf);
      
      //    
      SendMessage(hwndEdit, EM_GETCHARFORMAT, TRUE, (LPARAM)&cf);

      //   ,  
      //    
      memset(&chfnt, 0, sizeof(chfnt));
      memset(&lf, 0, sizeof(lf));

      //   
      hDC = GetDC(hWnd);

      //        ,
      //     
      lf.lfItalic    = (BOOL)(cf.dwEffects & CFE_ITALIC);
      lf.lfUnderline = (BOOL)(cf.dwEffects & CFE_UNDERLINE);

      //    TWIPS-  .
      //   ,  
      //     
      //   
      lf.lfHeight = - cf.yHeight/20;

      //  ,   
      lf.lfCharSet = ANSI_CHARSET;

      //  ,   
      lf.lfQuality = DEFAULT_QUALITY;

      //   
      lf.lfPitchAndFamily = cf.bPitchAndFamily;

      //   
      lstrcpy(lf.lfFaceName, cf.szFaceName);

      //       ,
      //      
      //  
      if(cf.dwEffects & CFE_BOLD)
        lf.lfWeight = FW_BOLD;
      else
        lf.lfWeight = FW_NORMAL;

      //      
      chfnt.lStructSize = sizeof(chfnt);
      chfnt.Flags = CF_SCREENFONTS | CF_INITTOLOGFONTSTRUCT;
      chfnt.hDC         = hDC;
      chfnt.hwndOwner   = hWnd;
      chfnt.lpLogFont   = &lf;
      chfnt.rgbColors   = RGB(0,0,0);
      chfnt.nFontType   = SCREEN_FONTTYPE;

      //      
      //  
      if(ChooseFont(&chfnt))
      {
        //      dwEffects
        cf.dwMask = CFM_BOLD | CFM_FACE | CFM_ITALIC |
          CFM_UNDERLINE | CFM_SIZE | CFM_OFFSET;

        //   TWIPS-
        cf.yHeight = - lf.lfHeight * 20;

        //   dwEffects 
        cf.dwEffects = 0;
        if(lf.lfUnderline)
          cf.dwEffects |= CFE_UNDERLINE;

        if(lf.lfWeight == FW_BOLD)
          cf.dwEffects |= CFE_BOLD;

        if(lf.lfItalic)
          cf.dwEffects |= CFE_ITALIC;

        //   
        cf.bPitchAndFamily = lf.lfPitchAndFamily;

        //    
        lstrcpy(cf.szFaceName, lf.lfFaceName);

        //    
        SendMessage(hwndEdit, EM_SETCHARFORMAT, 
          SCF_SELECTION, (LPARAM)&cf);
      }
      
      //   
      ReleaseDC(hWnd, hDC);

	  return 0L;
	  break;
    }

	//      
    //    Rich Edit
	case ID_FORMAT_PARAGRAPH_LEFT:
    {
	  pf.cbSize = sizeof(pf);
      pf.dwMask = PFM_ALIGNMENT;
      pf.wAlignment = PFA_LEFT;

      //     
      SendMessage(hwndEdit, EM_SETPARAFORMAT, 0, (LPARAM)&pf);

	  return 0L;
	  break;
    }

	//      
    //    Rich Edit
	case ID_FORMAT_PARAGRAPH_RIGHT:
    {
	  pf.cbSize = sizeof(pf);
      pf.dwMask = PFM_ALIGNMENT;
      pf.wAlignment = PFA_RIGHT;
      SendMessage(hwndEdit, EM_SETPARAFORMAT, 0, (LPARAM)&pf);
	  return 0L;
	  break;
    }

	//    
	case ID_FORMAT_PARAGRAPH_CENTER:
    {
	  pf.cbSize = sizeof(pf);
      pf.dwMask = PFM_ALIGNMENT;
      pf.wAlignment = PFA_CENTER;
      SendMessage(hwndEdit, EM_SETPARAFORMAT, 0, (LPARAM)&pf);
	  return 0L;
	  break;
    }

	//     Edit
	case ID_EDIT_UNDO:
      SendMessage(hwndEdit, EM_UNDO, 0, 0L);
	  return 0L;
	  break;

	case ID_EDIT_CUT:
      SendMessage(hwndEdit, WM_CUT, 0, 0L);
	  return 0L;
	  break;

	case ID_EDIT_COPY:
      SendMessage(hwndEdit, WM_COPY, 0, 0L);
	  return 0L;
	  break;

	case ID_EDIT_PASTE:
      SendMessage(hwndEdit, WM_PASTE, 0, 0L);
	  return 0L;
	  break;

	case ID_EDIT_DELETE:
      SendMessage(hwndEdit, WM_CLEAR, 0, 0L);
	  return 0L;
	  break;

	//   ,    
    //   Rich Edit
	case ID_EDIT_SELECTALL:
    {
      CHARRANGE charr;

      charr.cpMin = 0;  //  ... 
      charr.cpMax = -1; // ...    
      
      SendMessage(hwndEdit, EM_EXSETSEL, 0, (LPARAM)&charr);
	  return 0L;
	  break;
    }

	//      
    //   
	case ID_FILE_NEW:
      SetWindowText(hwndEdit,"\0");
	  return 0L;
	  break;

	case ID_FILE_OPEN:
      FileOpen(hWnd);   //    
	  return 0L;
	  break;
	
	case ID_FILE_SAVEAS:
      FileSaveAs(hWnd); //    
	  return 0L;
	  break;

	case ID_FILE_PRINT:
      FilePrint();      //  
	  return 0L;
	  break;

	case ID_FILE_EXIT:  
	  PostQuitMessage(0); //   
	  return 0L;
	  break;
	  
  	case ID_HELP_ABOUT:
      MessageBox(hWnd, 
        "Rich Text Editor RtfPad, v.1.0\n"
        "(C) Alexandr Frolov, 1995\n"
        "Email: frolov@glas.apc.org",
        szAppTitle, MB_OK | MB_ICONINFORMATION);
	  return 0L;
	  break;

  	default:
	  break;
  }
  return FORWARD_WM_COMMAND(hWnd, id, hwndCtl, codeNotify,
    DefWindowProc);
}

// -----------------------------------------------------
//  WndProc_OnSize       
// -----------------------------------------------------
#pragma warning(disable: 4098)
void WndProc_OnSize(HWND hwnd, UINT state, int cx, int cy)
{
  MoveWindow(hwndEdit, 0, 0, cx, cy, TRUE);
  return FORWARD_WM_SIZE(hwnd, state, cx, cy, DefWindowProc);
}

// -----------------------------------------------------
//  WndProc_OnSetFocus
// -----------------------------------------------------
#pragma warning(disable: 4098)
void WndProc_OnSetFocus(HWND hwnd, HWND hwndOldFocus)
{
  //       
  //  ,     
  //   Rich Edit
  SetFocus(hwndEdit);
  return FORWARD_WM_SETFOCUS(hwnd, hwndOldFocus, DefWindowProc);
}

// -----------------------------------------------------
//  FileSaveAs
// -----------------------------------------------------
void FileSaveAs(HWND hwnd)
{
  OPENFILENAME ofn;
  char szFile[256] = "untitled.rtf";
  char szDirName[512];
  char szFileTitle[256];
  
  //       
  //   rtf, txt,   
  char szFilter[256] = 
    "Rich Text Files\0*.rtf\0Text Files\0*.txt\0"
    "Any Files\0*.*\0";

  HFILE      hFile;
  OFSTRUCT   of;
  EDITSTREAM es;

  memset(&ofn, 0, sizeof(OPENFILENAME));
    
  //     
  GetCurrentDirectory(sizeof(szDirName), szDirName);
  
  //      
  ofn.lStructSize     = sizeof(OPENFILENAME);
  ofn.hwndOwner       = hwnd;
  ofn.lpstrFilter     = szFilter;
  ofn.lpstrInitialDir = szDirName;
  ofn.nFilterIndex    = 1;
  ofn.lpstrFile       = szFile;
  ofn.nMaxFile        = sizeof(szFile);
  ofn.lpstrFileTitle  = szFileTitle;
  ofn.nMaxFileTitle   = sizeof(szFileTitle);
  ofn.lpstrDefExt = "rtf";
  ofn.Flags = OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY;

  //     , 
  //    
  if(GetSaveFileName(&ofn))
  {
    //   ,     
    // 
    if (*ofn.lpstrFile)
    {
      hFile = OpenFile(ofn.lpstrFile, &of, OF_CREATE);

      //     ,
      //    
      es.dwCookie    = (DWORD)hFile; 
      es.dwError     = 0; 
      es.pfnCallback = SaveCallback; 
            
      //    rtf,   
      // rtf-.      
      //   
      _strupr(&ofn.lpstrFile[ofn.nFileExtension]); 
   
      if(!strncmp(&ofn.lpstrFile[ofn.nFileExtension], "RTF", 3))
         SendMessage(hwndEdit, EM_STREAMOUT, SF_RTF, (LPARAM)&es);
      else
         SendMessage(hwndEdit, EM_STREAMOUT, SF_TEXT, (LPARAM)&es);

      //  
      _lclose(hFile);

      //     
      //  
      SendMessage(hwndEdit, EM_SETMODIFY, FALSE, 0L);
    }
  }
}

// -----------------------------------------------------
//  SaveCallback
// -----------------------------------------------------
DWORD CALLBACK 
SaveCallback(DWORD dwCookie, LPBYTE   pbBuff,
  LONG cb, LONG *pcb)
{
  //      cb 
  cb = _lwrite((HFILE)dwCookie, pbBuff, cb);
  *pcb = cb;
  return 0;
}

// -----------------------------------------------------
//  FileOpen
// -----------------------------------------------------
void FileOpen(HWND hwnd)
{
  OPENFILENAME ofn;
  char szFile[256];
  char szDirName[256];
  char szFileTitle[256];
  char szFilter[256] = 
    "Rich Text Files\0*.rtf\0Text Files\0*.txt\0"
    "Any Files\0*.*\0";

  HFILE      hFile;
  OFSTRUCT   of;
  EDITSTREAM es;

  memset(&ofn, 0, sizeof(OPENFILENAME));
  GetCurrentDirectory(sizeof(szDirName), szDirName);
  szFile[0] = '\0';

  //      
  ofn.lStructSize     = sizeof(OPENFILENAME);
  ofn.hwndOwner       = hwnd;
  ofn.lpstrFilter     = szFilter;
  ofn.lpstrInitialDir = szDirName;
  ofn.nFilterIndex    = 1;
  ofn.lpstrFile       = szFile;
  ofn.nMaxFile        = sizeof(szFile);
  ofn.lpstrFileTitle  = szFileTitle;
  ofn.nMaxFileTitle   = sizeof(szFileTitle);
  ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

  //     , 
  //    
  if(GetOpenFileName(&ofn))
  {
    //   ,    
    if (*ofn.lpstrFile)
    {
      hFile = OpenFile(ofn.lpstrFile, &of, OF_READ);
      
      //     ,
      //    
      es.dwCookie    = (DWORD)hFile; 
      es.dwError     = 0; 
      es.pfnCallback = OpenCallback; 

      //    rtf,   
      // rtf-.      
      //   
      _strupr(&ofn.lpstrFile[ofn.nFileExtension]); 
      if(!strncmp(&ofn.lpstrFile[ofn.nFileExtension], "RTF", 3))
        SendMessage(hwndEdit, EM_STREAMIN, SF_RTF, (LPARAM)&es);
      else
        SendMessage(hwndEdit, EM_STREAMIN, SF_TEXT, (LPARAM)&es);

      //  
      _lclose(hFile);

      //     
      //  
      SendMessage(hwndEdit, EM_SETMODIFY, FALSE, 0L);
    }
  }
}

// -----------------------------------------------------
//  OpenCallback
// -----------------------------------------------------
DWORD CALLBACK 
OpenCallback(DWORD dwCookie, LPBYTE   pbBuff,
  LONG cb, LONG *pcb)
{
  //      cb 
  *pcb = _lread((HFILE)dwCookie, pbBuff, cb);
  if(*pcb <= 0)
    *pcb = 0;
  return 0;
}

// -----------------------------------------------------
//  FilePrint
// -----------------------------------------------------
void FilePrint(void)
{
  FORMATRANGE fr;
  DOCINFO docInfo;
  LONG lLastChar, lTextSize;
  PRINTDLG pd;
  int nRc;
  HDC hPrintDC;

  //    PRITDLG
  memset(&pd, 0, sizeof(pd));
  pd.lStructSize = sizeof(PRINTDLG);
  pd.hwndOwner   = hwndEdit;
  pd.hInstance   = (HANDLE)hInst;
  pd.Flags       = PD_RETURNDC | PD_NOPAGENUMS | 
    PD_NOSELECTION | PD_PRINTSETUP | PD_ALLPAGES;
  pd.nFromPage   = 0xffff;
  pd.nToPage     = 0xffff;
  pd.nMinPage    = 0;
  pd.nMaxPage    = 0xffff;
  pd.nCopies     = 1;

  //     , 
  //   
  if(PrintDlg(&pd) == TRUE)
  {
    hPrintDC = pd.hDC;

    //    FORMATRANGE 
    memset(&fr, 0, sizeof(fr));
  
    //      
    // ,    PrintDlg
    fr.hdc = fr.hdcTarget = hPrintDC;

    //   
    fr.chrg.cpMin = 0;     
    fr.chrg.cpMax = -1;
  
    //     TWIPS-
    fr.rcPage.top = 0;
    fr.rcPage.left = 0;
    fr.rcPage.right  = MulDiv(GetDeviceCaps(hPrintDC, PHYSICALWIDTH), 
      1440, GetDeviceCaps(hPrintDC, LOGPIXELSX));

    fr.rcPage.bottom = MulDiv(GetDeviceCaps(hPrintDC, PHYSICALHEIGHT), 
      1440, GetDeviceCaps(hPrintDC, LOGPIXELSY));
    fr.rc = fr.rcPage;

    //  
    if(fr.rcPage.right > 2*3*1440/4+1440)
      fr.rc.right -= (fr.rc.left = 3*1440/4);
    if(fr.rcPage.bottom > 3*1440)
      fr.rc.bottom -= (fr.rc.top = 1440);
    
    //    DOCINFO
    memset(&docInfo, 0, sizeof(DOCINFO));
    docInfo.cbSize = sizeof(DOCINFO);
    docInfo.lpszOutput = NULL;
    docInfo.lpszDocName = "RtfPad document";

    //   
    nRc = StartDoc(hPrintDC, &docInfo);
    
    //   ,     
    //  
    if(nRc < 0)
    {
      char szErr[128];
      DWORD dwErr = GetLastError();
      wsprintf(szErr, "Print Error %ld", dwErr);

      MessageBox(NULL, szErr,
        szAppTitle, MB_OK | MB_ICONEXCLAMATION);
      
      DeleteDC(hPrintDC);
      return;
    }

    //   
    StartPage(hPrintDC);

    lLastChar = 0;

    //     
    lTextSize = SendMessage(hwndEdit, WM_GETTEXTLENGTH, 0, 0);

    //     
    while (lLastChar < lTextSize)
    {
      //       
      lLastChar = SendMessage(hwndEdit, EM_FORMATRANGE, TRUE, 
        (LPARAM) &fr);
        
      if(lLastChar < lTextSize)
      {
        //    
        EndPage(hPrintDC);
        
        //   
        StartPage(hPrintDC);    
        fr.chrg.cpMin = lLastChar;
        fr.chrg.cpMax = -1;
      }
    }

    //  ,    
    //   Rich Edit
    SendMessage(hwndEdit, EM_FORMATRANGE, TRUE, (LPARAM)NULL);

    //   
    EndPage(hPrintDC);

    //   
    EndDoc(hPrintDC);

    //   
    DeleteDC(hPrintDC);
  }
}
