#define STRICT
#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include "resource.h"
#include "afxres.h"
#include "semmdi.h"

//   
char const szFrameClassName[] = "MDISemAppClass";
char const szChildClassName[] = "MDISemChildAppClass";

//  
char const szWindowTitle[] = 
  "Multithread MDI Application with Semaphores";

HINSTANCE hInst;

HWND hwndFrame;  //  Frame Window
HWND hwndClient; //  Client Window
HWND hwndChild;  //  Child Window

// ,      
typedef struct _CHILD_WINDOW_TAG
{
  //   
  BOOL fActive;

  //   
  BOOL fWaiting;
  
  //      
  CRITICAL_SECTION csChildWindowPaint; 

  //  
  HANDLE hThread;
} CHILD_WINDOW_TAG;

typedef CHILD_WINDOW_TAG *LPCHILD_WINDOW_TAG;

//      
HANDLE hSemaphore;

//  
char const lpSemaphoreName[] =
 "$MyVerySpecialSemaphoreName$For$SemMDI$Application$";

// =====================================
//  WinMain
// =====================================
int APIENTRY 
WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
        LPSTR lpCmdLine, int nCmdShow)
{
  MSG  msg;          //     
  hInst = hInstance; //   

  //  
  if(!InitApp(hInstance))
    return FALSE;

  //     
  // ,  
  hSemaphore = CreateSemaphore(NULL, 2, 2, lpSemaphoreName);
  if(hSemaphore == NULL)
  {
    MessageBox(NULL,
      "   ",
       szWindowTitle, MB_OK | MB_ICONEXCLAMATION);
    return FALSE;
  }
  
  //     - Frame Window
  hwndFrame = CreateWindow(
    szFrameClassName,    //   
    szWindowTitle,       //  
    WS_OVERLAPPEDWINDOW, //  
    CW_USEDEFAULT, 0,    //    
    CW_USEDEFAULT, 0,    // ,   
    0,                   //   
    0,                   //  
    hInstance,           //  
    NULL);   //    

  //     ,  
  if(!hwndFrame)
    return FALSE;

  //   
  ShowWindow(hwndFrame, nCmdShow);
  UpdateWindow(hwndFrame);

  //    
  while(GetMessage(&msg, NULL, 0, 0))
  {
    //   MDI-
    if(!TranslateMDISysAccel(hwndClient, &msg))
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }
  }
  
  CloseHandle(hSemaphore);
  return msg.wParam;
}

// =====================================
//  InitApp
//    
// =====================================

BOOL InitApp(HINSTANCE hInstance)
{
  ATOM aWndClass; //    
  WNDCLASS wc;    //   

  //      
  // (  Frame Window)
  memset(&wc, 0, sizeof(wc));
  wc.lpszMenuName  = "APP_MENU";
  wc.style         = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc   = (WNDPROC)FrameWndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon(hInstance, "APP_ICON");
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)(COLOR_APPWORKSPACE + 1);
  wc.lpszClassName = (LPSTR)szFrameClassName;
  aWndClass = RegisterClass(&wc);

  if(!aWndClass)
    return FALSE;

  //     
  //  Document Window
  memset(&wc, 0, sizeof(wc));
  wc.lpszMenuName  = 0;
  wc.style         = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc   = (WNDPROC)ChildWndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon(hInstance, "APPCLIENT_ICON");
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
  wc.lpszClassName = (LPSTR)szChildClassName;
  aWndClass = RegisterClass(&wc);

  if(!aWndClass)
    return FALSE;

  return TRUE;
}

// =====================================
//  FrameWndProc
// =====================================

LRESULT CALLBACK 
FrameWndProc(HWND hwnd, UINT msg, 
             WPARAM wParam, LPARAM lParam)
{
  HWND   hwndChild;
  HANDLE hThread;
  DWORD  dwIDThread;
  CHAR   szBuf[255];
  LPCHILD_WINDOW_TAG lpMyWndTag;
  
  //     Client Window
  CLIENTCREATESTRUCT clcs;

  //      
  //   
  LPCHILD_WINDOW_TAG lpTag;

  switch (msg)
  {
    //    Frame Window 
    //  Client Window,    
    //   Document Window
    case WM_CREATE:
    {
      clcs.hWindowMenu = GetSubMenu(GetMenu(hwnd), 2);
      clcs.idFirstChild = 500;
      
      //   Client Window 
      hwndClient = CreateWindow(
        "MDICLIENT",    //   
        NULL,           //  
        WS_CHILD | WS_CLIPCHILDREN | WS_VISIBLE | 
          WS_HSCROLL | WS_VSCROLL,  
        0, 0, 0, 0,
        hwnd,           //   
        (HMENU)1,       //  
        hInst,          //  
        (LPSTR)&clcs);  //    

      break;
    }

    //      
    case WM_COMMAND:
    {
      switch (wParam)
      {
        //    Document Window
        case CM_FILENEW:
        {
          hwndChild = CreateMDIWindow(
            (LPSTR)szChildClassName,              //  
            "MDI Child Window",                   //  
            0,                              //  
            CW_USEDEFAULT, CW_USEDEFAULT,   //   
            CW_USEDEFAULT, CW_USEDEFAULT,   //    Document Window
            hwndClient,
            hInst,                      //  
            0);                         //  

          //    ,   
          //   
          lpTag = malloc(sizeof(CHILD_WINDOW_TAG));
  
          //   
          lpTag->fActive = 1;

          //   
          InitializeCriticalSection(&(lpTag->csChildWindowPaint));
          
          //     
          //  
          SetWindowLong(hwndChild, GWL_USERDATA, (LONG)lpTag);

          //     
          hThread = CreateThread(NULL, 0, 
            (LPTHREAD_START_ROUTINE)ThreadRoutine,
            (LPVOID)hwndChild, 0,(LPDWORD)&dwIDThread);

          if(hThread == NULL)
          {
            MessageBox(hwnd,"   ",
	           szWindowTitle, MB_OK | MB_ICONEXCLAMATION);
          }

          //    
          lpTag->hThread = hThread;
          
          //     
          //  
          sprintf(szBuf, "Thread ID = %lX", dwIDThread);
          SetWindowText(hwndChild, szBuf);
   
          break;
        }

        //    
        //  
        case ID_SEMAPHORE_INCREMENT:
        {
          ReleaseSemaphore(hSemaphore, 1, NULL);
          break;
        }
        
        //   Document Window    
        case CM_WINDOWTILE:
        {
          SendMessage(hwndClient, WM_MDITILE, 0, 0);
          break;
        }

        //   Document Window  
        case CM_WINDOWCASCADE:
        {
          SendMessage(hwndClient, WM_MDICASCADE, 0, 0);
          break;
        }

        //     
        // Document Window     Client Window
        case CM_WINDOWICONS:
        {
          SendMessage(hwndClient, WM_MDIICONARRANGE, 0, 0);
          break;
        }

        //    Document Window
        case CM_WINDOWCLOSEALL:
        {
          HWND hwndTemp;

          ShowWindow(hwndClient, SW_HIDE);

          while(TRUE) 
          {
            //     
            //   Client Window
            hwndTemp = GetWindow(hwndClient, GW_CHILD);
            //     ,   
            if(!hwndTemp)
              break;

            //  -
            while(hwndTemp && GetWindow(hwndTemp, GW_OWNER))
              hwndTemp = GetWindow(hwndTemp, GW_HWNDNEXT);

            //    Document Window
            if(hwndTemp)
            {
              //  ,   
              lpMyWndTag = 
                 (LPCHILD_WINDOW_TAG)GetWindowLong(
                 hwndTemp, GWL_USERDATA);
              lpMyWndTag->fActive = 0;  

              //  ,    
              //   
              SendMessage(hwndClient, WM_MDIDESTROY, 
                (WPARAM)hwndTemp, 0);
            }
            else
              break;
          }

          //   Client Window
          ShowWindow(hwndClient, SW_SHOW);

          break;
        }

        //    
        case ID_PRIORITYCLASS_REALTIME:
        {
          SetPriorityClass(GetCurrentProcess(),
            REALTIME_PRIORITY_CLASS);
          break;
        }
        case ID_PRIORITYCLASS_HIGH:
        {
          SetPriorityClass(GetCurrentProcess(),
            HIGH_PRIORITY_CLASS);
          break;
        }
        case ID_PRIORITYCLASS_NORMAL:
        {
          SetPriorityClass(GetCurrentProcess(),
            NORMAL_PRIORITY_CLASS);
          break;
        }
        case ID_PRIORITYCLASS_IDLE:
        {
          SetPriorityClass(GetCurrentProcess(),
            IDLE_PRIORITY_CLASS);
          break;
        }

        case CM_HELPABOUT:
        {
	        MessageBox(hwnd,
	          "  \n"
            " MDI-\n"
            "(C) Alexandr Frolov, 1996\n"
            "Email: frolov@glas.apc.org",
            szWindowTitle, MB_OK | MB_ICONINFORMATION);
          break;
        }

        //   
        case CM_FILEEXIT:
        {
          DestroyWindow(hwnd);
          break;
        }

        default:
          break;
      }

      //     
      // Document Window
      hwndChild =
        (HWND)LOWORD(SendMessage(hwndClient, 
        WM_MDIGETACTIVE, 0, 0l));

      //   ,    WM_COMMAND
      if(IsWindow(hwndChild))
        SendMessage(hwndChild, WM_COMMAND, wParam, lParam);

      return DefFrameProc(
        hwnd, hwndClient, msg, wParam, lParam);
    }

    case WM_DESTROY:
    {
      PostQuitMessage(0);
      break;
    }

    default:
      break;
  }
  return DefFrameProc(hwnd, hwndClient, msg, wParam, lParam);
}

// =====================================
//  ChildWndProc
// =====================================

LRESULT CALLBACK
ChildWndProc(HWND hwnd, UINT msg, 
             WPARAM wParam, LPARAM lParam)
{
  HDC hdc;
  PAINTSTRUCT ps;
  RECT rc;
  LPCHILD_WINDOW_TAG lpMyWndTag;
  HMENU hmenuPopup;
  POINT pt;
  CHAR szBuf[256];

  switch (msg)
  {
    case WM_PAINT:
    {
      //     
      lpMyWndTag = 
        (LPCHILD_WINDOW_TAG)GetWindowLong(hwnd, GWL_USERDATA);

      //    
      EnterCriticalSection(&(lpMyWndTag->csChildWindowPaint));

      //     
      hdc = BeginPaint(hwnd, &ps);
      GetClientRect(hwnd, &rc);

      if(lpMyWndTag->fWaiting)
        DrawText(hdc, "Waiting...", -1, &rc,
          DT_SINGLELINE | DT_CENTER | DT_VCENTER);
      else
        DrawText(hdc, "Child Window. Running", -1, &rc,
          DT_SINGLELINE | DT_CENTER | DT_VCENTER);

      EndPaint(hwnd, &ps);

      //    
      LeaveCriticalSection(&(lpMyWndTag->csChildWindowPaint));
      break;
    }

    case WM_CLOSE:
    {
      //    
      lpMyWndTag = 
        (LPCHILD_WINDOW_TAG)GetWindowLong(hwnd, GWL_USERDATA);

      //       
      // ,   
      if(lpMyWndTag->fWaiting)
        return 0;
      
      //     , 
      //     
      else
        lpMyWndTag->fActive = 0;  
      
      break;
    }

    //       
    //   ,   
    case WM_RBUTTONDOWN:
    {
      pt.x = LOWORD(lParam);
      pt.y = HIWORD(lParam);
      ClientToScreen(hwnd, &pt);

      hmenuPopup = GetSubMenu(
        LoadMenu(hInst, "IDR_POPUPMENU"), 0);
      TrackPopupMenu(hmenuPopup, 
        TPM_CENTERALIGN | TPM_LEFTBUTTON,
        pt.x, pt.y, 0, hwnd, NULL);
      DestroyMenu(hmenuPopup);
      break;
    }

    //  ,    
    case WM_COMMAND:
    {
      switch (wParam)
      {
        //   
        case ID_THREADCONTROL_SUSPEND:
        {
          lpMyWndTag = 
            (LPCHILD_WINDOW_TAG)GetWindowLong(hwnd, GWL_USERDATA);
          
          //    
          EnterCriticalSection(&(lpMyWndTag->csChildWindowPaint));

          SuspendThread(lpMyWndTag->hThread);

		      //    
          LeaveCriticalSection(&(lpMyWndTag->csChildWindowPaint));

          break;
        }
        
        //   
        case ID_THREADCONTROL_RESUME:
        {
   
          lpMyWndTag = 
            (LPCHILD_WINDOW_TAG)GetWindowLong(hwnd, GWL_USERDATA);
          ResumeThread(lpMyWndTag->hThread);
          break;
        }

        //   
        case ID_THREADCONTROL_PRIORITYLOWEST:
        {
          lpMyWndTag = 
            (LPCHILD_WINDOW_TAG)GetWindowLong(hwnd, GWL_USERDATA);
          SetThreadPriority(lpMyWndTag->hThread, 
            THREAD_PRIORITY_LOWEST);
          break;
        }
        case ID_THREADCONTROL_PRIORITYNORMAL:
        {
          lpMyWndTag = 
            (LPCHILD_WINDOW_TAG)GetWindowLong(hwnd, GWL_USERDATA);
          SetThreadPriority(lpMyWndTag->hThread, 
            THREAD_PRIORITY_NORMAL);
          break;
        }
        case ID_THREADCONTROL_PRIORITYHIGHEST:
        {
          lpMyWndTag = 
            (LPCHILD_WINDOW_TAG)GetWindowLong(hwnd, GWL_USERDATA);
          SetThreadPriority(lpMyWndTag->hThread, 
            THREAD_PRIORITY_HIGHEST);
          break;
        }

        //     
        case ID_THREADCONTROL_GETPRIORITY:
        {
          lpMyWndTag = 
            (LPCHILD_WINDOW_TAG)GetWindowLong(hwnd, GWL_USERDATA);
          
          strcpy(szBuf, "Thread priority: ");
          
          switch (GetThreadPriority(lpMyWndTag->hThread))
          {
            case THREAD_PRIORITY_LOWEST:
            {
              strcat(szBuf, "THREAD_PRIORITY_LOWEST");
              break;
            }
            case THREAD_PRIORITY_BELOW_NORMAL:
            {
              strcat(szBuf, "THREAD_PRIORITY_BELOW_NORMAL");
              break;
            }
            case THREAD_PRIORITY_NORMAL:
            {
              strcat(szBuf, "THREAD_PRIORITY_NORMAL");
              break;
            }
            case THREAD_PRIORITY_ABOVE_NORMAL:
            {
              strcat(szBuf, "THREAD_PRIORITY_ABOVE_NORMAL");
              break;
            }
            case THREAD_PRIORITY_HIGHEST:
            {
              strcat(szBuf, "THREAD_PRIORITY_HIGHEST");
              break;
            }
          }

	        MessageBox(hwnd, szBuf,
            szWindowTitle, MB_OK | MB_ICONINFORMATION);
          
          break;
        }

        //  
        case ID_THREADCONTROL_KILLTHREAD:
        {
          lpMyWndTag = 
            (LPCHILD_WINDOW_TAG)GetWindowLong(hwnd, GWL_USERDATA);
          TerminateThread(lpMyWndTag->hThread, 5);
          break;
        }
        
        //   
        case ID_THREADCONTROL_CLOSEWINDOW:
        {
          lpMyWndTag = 
            (LPCHILD_WINDOW_TAG)GetWindowLong(hwnd, GWL_USERDATA);

          //       
          // ,      
          // ,   
          if(!lpMyWndTag->fWaiting)
          {
            SendMessage(hwndClient, WM_MDIDESTROY, 
                (WPARAM)hwnd, 0);

            lpMyWndTag->fActive = 0;  
          }
          break;
        }
        default:
          break;
      }
      break;
    }
    default:
      break;
  }
  return DefMDIChildProc(hwnd, msg, wParam, lParam);
}

// =====================================
//  ThreadRoutine
// ,    
//  
// =====================================

DWORD ThreadRoutine(HWND hwnd) 
{
  LONG lThreadWorking = 1L;
  HDC hDC;
  RECT rect;
  LONG xLeft, xRight, yTop, yBottom;
  short nRed, nGreen, nBlue;
  HBRUSH hBrush, hOldBrush;
  LPCHILD_WINDOW_TAG lpMyWndTag;

  //       
  lpMyWndTag = 
    (LPCHILD_WINDOW_TAG)GetWindowLong(hwnd, GWL_USERDATA);

  //   ,   
  if(lpMyWndTag == NULL)
  {
    return 0;
  }

  //    
  lpMyWndTag->fWaiting = TRUE;

  //   .   
  //  ,   
  if(WAIT_FAILED == 
    WaitForSingleObject(hSemaphore, INFINITE))
  {
    return 0;
  }

  //    
  lpMyWndTag->fWaiting = FALSE;

  srand((unsigned int)hwnd);
  
  while(TRUE) 
  {
    if(!lpMyWndTag->fActive)
      break;

    EnterCriticalSection(&(lpMyWndTag->csChildWindowPaint));

    hDC = GetDC(hwnd);
    nRed   = rand() % 255;
    nGreen = rand() % 255;
    nBlue  = rand() % 255;
    
    GetWindowRect(hwnd, &rect);
    xLeft   = rand() % (rect.left   + 1);
    xRight  = rand() % (rect.right  + 1);
    yTop    = rand() % (rect.top    + 1);
    yBottom = rand() % (rect.bottom + 1);

    hBrush = CreateSolidBrush(RGB(nRed, nGreen, nBlue));
    hOldBrush = SelectObject(hDC, hBrush);

    Ellipse(hDC, min(xLeft, xRight), min(yTop, yBottom),
                 max(xLeft, xRight), max(yTop, yBottom));

    SelectObject(hDC, hOldBrush);
    DeleteObject(hBrush);
    ReleaseDC(hwnd, hDC);
    
    LeaveCriticalSection(&(lpMyWndTag->csChildWindowPaint));
    Sleep(1);
  }

  DeleteCriticalSection(&(lpMyWndTag->csChildWindowPaint));
  free(lpMyWndTag);
  
  //     
  //    ,  
  //  ,    
  if(hSemaphore != NULL)
    ReleaseSemaphore(hSemaphore, 1, NULL);

  return 0;
}

