// ----------------------------------------
//  SYSPAL
//    
// ----------------------------------------

#define STRICT
#include <windows.h>
#include <windowsx.h>
#include <mem.h>

#define PALETTESIZE   256

BOOL InitApp(HINSTANCE);
LRESULT CALLBACK _export WndProc(HWND, UINT, WPARAM, LPARAM);
BOOL PaletteInfo(void);

char const szClassName[]   = "SysPalClass";
char const szWindowTitle[] = "System Palette";

short cxClient, cyClient;
HPALETTE hPal, hOldPalette;
NPLOGPALETTE  pLogPal;

// =====================================
//  WinMain
// =====================================
#pragma argsused

int PASCAL
WinMain(HINSTANCE hInstance, 
	HINSTANCE hPrevInstance,
	LPSTR     lpszCmdLine, 
	int       nCmdShow)
{
  MSG  msg;   //     
  HWND hwnd;  //    

  if(!hPrevInstance)
    if(!InitApp(hInstance))
      return FALSE;

  if(!PaletteInfo())
    return FALSE;

  hwnd = CreateWindow(
    szClassName,         //   
    szWindowTitle,       //  
    WS_OVERLAPPEDWINDOW, //  
    CW_USEDEFAULT,       //    
    CW_USEDEFAULT,       // ,   
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    0, 0, hInstance, NULL);      
                       
  if(!hwnd)
    return FALSE;

  ShowWindow(hwnd, nCmdShow);
  UpdateWindow(hwnd);

  while(GetMessage(&msg, 0, 0, 0))
  {
    DispatchMessage(&msg);
  }
  return msg.wParam;
}

// =====================================
//  InitApp
//    
// =====================================

BOOL
InitApp(HINSTANCE hInstance)
{
  ATOM aWndClass; //    
  WNDCLASS wc;    //   
                  //  
  //       
  memset(&wc, 0, sizeof(wc));

  wc.lpszMenuName  = NULL;
  wc.style         = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc   = (WNDPROC) WndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
  wc.lpszClassName = (LPSTR)szClassName;

  //  
  aWndClass = RegisterClass(&wc);

  return (aWndClass != 0);
}

// =====================================
//  WndProc
// =====================================

LRESULT CALLBACK _export
WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  HDC hdc;
  PAINTSTRUCT ps;

  switch (msg)
  {
    case WM_CREATE:
    {
      int i;

      //    
      pLogPal = (NPLOGPALETTE) LocalAlloc(LMEM_FIXED,
        (sizeof (LOGPALETTE) +
        (sizeof (PALETTEENTRY) * (PALETTESIZE))));

      //   
      pLogPal->palVersion = 0x300;
      pLogPal->palNumEntries = PALETTESIZE;

      //    PALETTEENTRY  
      // peRed  peGreen,   - peBlue  peFlags.
      //      PC_EXPLICIT
      for (i = 0;  i < PALETTESIZE;  i++)
      {
        pLogPal->palPalEntry[i].peBlue  = 0;
        *((PWORD) (&pLogPal->palPalEntry[i].peRed)) = i;
        pLogPal->palPalEntry[i].peFlags = PC_EXPLICIT;
      }

      //   
      hPal = CreatePalette((LPLOGPALETTE) pLogPal);
      return 0;
    }

    //     
    //      
    case WM_SIZE:
    {
      cxClient = LOWORD(lParam);
      cyClient = HIWORD(lParam);
      return 0;
    }

    //   
    case WM_PAINT:
    {
      RECT rc;
      int i, nWidth;
      HBRUSH hBrush;

      hdc = BeginPaint(hwnd, &ps);
      hOldPalette = SelectPalette(hdc, hPal, FALSE);
      RealizePalette(hdc);

      nWidth = 2;
      rc.left   = rc.top = 0;
      rc.right  = nWidth;
      rc.bottom = cyClient;

      for (i=0;  i < 256;  i++)
      {
        hBrush = CreateSolidBrush (PALETTEINDEX (i));
        FillRect (hdc, &rc, hBrush);
        rc.left   = rc.right;
        rc.right += nWidth;
        DeleteBrush(hBrush);
      }

      SelectPalette(hdc, hOldPalette, TRUE);
      EndPaint(hwnd, &ps);
      return 0;
    }

    case WM_PALETTECHANGED:
    {
       if (hwnd == (HWND) wParam)
         break;
    }

    case WM_QUERYNEWPALETTE:
    {
      HDC hdc;
      HPALETTE hOldPal;
      int nChanged;

      hdc = GetDC(hwnd);

      hOldPal = SelectPalette(hdc, hPal,
        (msg == WM_QUERYNEWPALETTE) ? FALSE : TRUE);

      nChanged = RealizePalette(hdc);
      SelectPalette(hdc, hOldPal, TRUE);

      ReleaseDC(hwnd, hdc);

      if(nChanged)
        InvalidateRect(hwnd, NULL, TRUE);

      return nChanged;
    }

    case WM_DESTROY:
    {
      DeletePalette(hPal);
      LocalFree(pLogPal);

      PostQuitMessage(0);
      return 0;
    }

    default:
      break;
  }
  return DefWindowProc(hwnd, msg, wParam, lParam);
}

// --------------------------------------------------------
//  PaletteInfo
//     
// --------------------------------------------------------

BOOL PaletteInfo(void)
{
  HDC hdc;
  int iRasterCaps;

  hdc = GetDC(NULL);
  iRasterCaps = GetDeviceCaps(hdc, RASTERCAPS);
  ReleaseDC(NULL, hdc);

  // ,    
  if(iRasterCaps & RC_PALETTE)
    return TRUE;
  else
    return FALSE;
}